;=======
;Alt-Tab
;=======

;The Alt-tab option can play havoc with the images in your game. When
;the user uses Alt-tab, your images may be erased, written over, or
;otherwise corrupted. Native Blitz protects you against this,
;automatically reloading all of your images when the user returns
;to the game.

;This feature may be added to a future version of the Extended B2D
;library. Until then, if you want to protect against Alt-tab problems
;you need to handle reloading images manually. To do that, you need
;to first detect whether an Alt-tab has occurred by using the
;WindowStatusBB() command. Then you need to free all of the previous
;images using the FreeAllImagesBB() command, and then reload them
;manually using all the usual load commands.

;This demo illustrates one way to handle Alt-tab using these commands.

Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if dll not in same directory)
GraphicsBB(800,600,16)

;Load images
Gosub LoadImages

;Main loop
While Not KeyDown(1)

	ClsBB()
	DrawBlockBB(mage,400,300)		
	DrawImageBB(mage,MouseCursorX-50,MouseCursorY-50)			
	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)
	FlipBB()
	
	;Check to see if user has used Alt-Tab
	Gosub CheckAltTab	
Wend

EndGraphicsBB()
End

;This routine loads all of our images and reloads them
;again if the user Alt-tabs out of the game and
;then returns.
.LoadImages
	AutoMidHandleBB(False)	
	mousePointer = LoadImageBB("graphics/red pointer.bmp")	
	MaskImageBB(mousePointer,255,255,255)
	AutoMidHandleBB(True)	
	mage = LoadImageBB("graphics/mage2.bmp")		
	MaskImageBB(mage,71,71,71)
	reloadNeeded = False
	Return

;This example subroutine checks to see if the user has minimized
;the game by using Alt-Tab. If so, it pauses the game and waits
;for the user to reactivate it. When that happens, it frees all
;existing images and then reloads them before returning to the
;main loop.
.CheckAltTab
	If WindowStatusBB() = 0 ;has game been minimized?
		While Not KeyDown(1) ;start pause loop
			If WindowStatusBB() = 1 ;has game been reactivated?
				FreeAllImagesBB() ;free the images previously loaded
				Gosub LoadImages ;reload our images
				Exit ;exit pause loop
			End If	
		Wend
	End If		
	Return	
	