;==============
;Copy 2D to 3D
;==============

;This demo shows how to copy 2D images onto 3D images and
;visa-versa.

;Please Note 
;-----------
;(1) You can use CopyImageBB, CreateImageBB, DrawImageBB, DrawBlockBB,
;    DrawImageRectBB and CopyRectBB on both 2D images and 3D images. 
;    They are slower with 3D images, however.

;(2) When drawing anything other than a 2D image on a 2D image,
;    you lose masking and will need to remask the resulting image.

;(3) When using CopyImageBB or CreateImageBB to create a 3d image,
;    remember that the dimensions of the copied or created image need
;    to be a power of 2.

;(4) Draw3DImageBB and Draw3DQuad only work with 3d images. Also
;    you can't use these two commands to draw on anything other than
;    the backbuffer. If you want to "capture" an image drawn with
;    these two commands, you will need to draw it to the backbuffer first
;    and then copy or save it from the back buffer using normal
;    2D commands like CopyRectBB or SaveBufferBB. If you want to do it 
;    in real time, here is how you would do it:

;	 ClsBB()
;    Draw3DImageBB()
;    CopyRectBB() ;or whatever
;	 [Don't flip!]
;    ClsBB() ;second ClsBB() covers up work space so it is never seen
;    on screen.
;    [do normal drawing stuff]
;    FlipBB()


Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16) ;3D-enabled not supported in 8-bit and 24-bit 

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)
mage = LoadImageBB("graphics/mage.bmp") ;normal 2D image
MaskImageBB(mage,71,71,71)
mage3D = Load3DImageBB("graphics/mage.bmp") ;3D image
MaskImageBB(mage3D,71,71,71)

;Create 4 images by using CopyImageBB. This shows that you can
;make a 2D image from a 3D image, a 3D image from a 2D image, etc.
mageCopy2Dto2D = CopyImageBB(mage)
mageCopy3Dto2D = CopyImageBB(mage3D)
mageCopy2Dto3D = CopyImageBB(mage,0,1)
mageCopy3Dto3D = CopyImageBB(mage3D,0,1)

;Create 4 images by using CreateImageBB and DrawImageBB
mageCreated2Dto2D = CreateImageBB(128,128)
SetBufferBB(mageCreated2Dto2D)
DrawImageBB(mage,0,0) ;draw 2D image on 2D image -- DRAWN MASKED!
SetBufferBB(BackBufferBB) ;always a good habit

mageCreated3Dto2D = CreateImageBB(128,128)
SetBufferBB(mageCreated3Dto2D)
DrawImageBB(mage3D,0,0) ;draw 3D image on 2D image
SetBufferBB(BackBufferBB)  ;always a good habit

mageCreated2Dto3D = CreateImageBB(128,128,0,1)
SetBufferBB(mageCreated2Dto3D)
DrawImageBB(mage,0,0) ;draw 2D image on 3D image
SetBufferBB(BackBufferBB) ;always a good habit

mageCreated3Dto3D = CreateImageBB(128,128,0,1)
SetBufferBB(mageCreated3Dto3D)
DrawImageBB(mage3D,0,0) ;draw 3D image on 3D image
SetBufferBB(BackBufferBB) ;always a good habit

;Create a 2D image and use DrawImageRectBB to draw a section
;of a 3D image onto the created 2D image
mageCreated2DRect1 = CreateImageBB(64,64)
SetBufferBB(mageCreated2DRect1)
DrawImageRectBB(mage3D,0,0,32,32,64,64)
SetBufferBB(BackBufferBB) ;always a good habit

;Create a 3D image and use DrawImageRectBB to draw a section
;of a 3D image onto the created 3D image
mageCreated3DRect1 = CreateImageBB(64,64,0,1)
SetBufferBB(mageCreated3DRect1)
DrawImageRectBB(mage3D,0,0,32,32,64,64)
SetBufferBB(BackBufferBB) ;always a good habit

;Create a 3D image and use CopyRectBB to draw a section
;of a 3D image onto the created 3D image
mageCreated3DRect2 = CreateImageBB(64,64,0,1)
CopyRectBB(32,32,64,64,0,0,mage3D,mageCreated3DRect2)
	
	
;Main loop
While Not KeyDown(1)
	ClsBB()

	;Draw 2D images that have been loaded/copied/created		
	DrawImageBB(mage,0,100)
	DrawImageBB(mageCopy2Dto2D,150,100)	
	DrawImageBB(mageCopy3Dto2D,300,100)		
	DrawImageBB(mageCreated2Dto2D,450,100)		
	DrawImageBB(mageCreated3Dto2D,600,100)				

	;Draw 3D images that have been loaded/copied/created	
	Draw3DImageBB(mage3D,0,300)	
	Draw3DImageBB(mageCopy2Dto3D,150,300)
	Draw3DImageBB(mageCopy3Dto3D,300,300)
	Draw3DImageBB(mageCreated2Dto3D,450,300)		
	Draw3DImageBB(mageCreated3Dto3D,600,300)

	;Draw some image segments that were created using CopyRect
	;and DrawImageRectBB
	DrawImageBB(mageCreated2DRect1,100,500)	
	Draw3DImageBB(mageCreated3DRect1,200,500)
	Draw3DImageBB(mageCreated3DRect2,300,500)		

	;Draw Mouse
	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)
	FlipBB()
Wend

DebugLog EndGraphicsBB()
End