;Editor's Note: This demo may require a slightly higher spec PC for best
;results, but it's very cool. For a simpler, but less cool, demo see
;my Draw3DQuad demo. -- Patrick
;=============

;Flag effect written to show off Patrick Lester's Extended2D lib.

;Original concept and math by Angus Arthur
;Coding and bugfixes by Simon Smith

;feel free to use the code, just give us all a mention.

;9th July 2003

Include "../../extendedblitzlib.bb"
dllFileName$ = "../../Blitz2DDll.dll"

; Looks great in higher resolutions
Global SCRwid=640, SCRhi=480, SCRdep=32

GraphicsBB(SCRwid,SCRhi,SCRdep)

; sine cos lookup tables, every little helps.
Dim sine#(360) : Dim cosine#(360)
For n=0 To 360
	sine(n)=Sin(n) : cosine(n)=Cos(n)
Next

; We need this to define the max # cells that can be stored to make up a flag
Const FLAGarea=5625 ; allows 75 by 75 cells across, or any combination that adds up to that.

; Define "flagpoles". Essentially modifiers for the way the flag animates
Const FLAGPOLEnone=0
Const FLAGPOLEtop=1
Const FLAGPOLEright=2
Const FLAGPOLEbottom=3	; okay, wierd but here for completeness
Const FLAGPOLEleft=4

; defines individual corners of cells, with the angle and radius to offset by
Type flagbit
	Field ang#,rad#
End Type

; a newtype to hold the flag, it's points and it's images (cells)
Type flagdefinition
	Field width#,xgap#  ; height in points, gap between points
	Field height#,ygap#	; height in points, gap between points
	Field cellsize		; square cellsize we have to use to fit into the flag. 16x16,32x32 ect...
	Field im[FLAGarea]	; an image handle for each cell of the flag
	Field totalim		; total images
	Field point.flagbit[FLAGarea]
End Type



; IMtemp:     Image to create flag from. 
; width:      number cells across the image (see comments within function)
; height:     number cells down the image
; xgap:       distance between points onscreen, defines the default size and "definition" for a flag.
; ygap:       as above but for y, obv. 
; inang:      Initial angle of flag's waving. 0-360
; rad:        Radius, the amount of deviation from the flat flag image
; colchange:  amount of wind across the flag
; rowchange:  amount of wind down the flag
Function CreateFlag.flagdefinition(IMtemp,width,height,xgap=16,ygap=16,inang#=0,rad#=15,colchange#=20,rowchange#=12,flagpole=False)
	If IMtemp=False Then Return Null
	
	; add one. see it's actually the number of points needed, but it's easier to think of in cells.
	width=width+1 : height=height+1

	cellsize=128	; largest desired cell size

	; Now a quick note about defining width and height in the function call.
	;   The actual pixel width/height of the image should divide by the width/height you send to the
	;   function and return an integer that can create square cells that the 3D card is compatible
	;   with, such, 32x32, 64x64 128x128 ect..
	;     e.g. a 640x400 image fits perfectly into 40x25 points (becomes 16x16 cells)
	; 		but not with 20x12 points (400 pixels can't divide into 12 cells matching a ^2 size)
	;       It's difficult to describe, but basically take your width/height in pixels and try dividing
	;	    into 64, 32, 16 or 8 and see what gives you a whole number. use that for width and height parameters.

	; figure out the largest sized "cell" we can use. Has to fit the power of 2.
	dwid=width-1	; desired width
	dhi=height-1	; desired height
	
	Repeat
		solved=False
		If cellsize*dwid<=ImageWidthBB(IMtemp) And cellsize*dhi<=ImageHeightBB(IMtemp)
			solved=True
		Else
			cellsize=cellsize/2
		End If
	Until solved=True
	
	; okay let's use the closest size we could find.
	dwid=cellsize*dwid
	dhi=cellsize*dhi
	
	
	; okay lets create a pointer to return
	p.flagdefinition=New flagdefinition
	
	p\cellsize=cellsize
	p\width=width : p\height=height
	p\xgap=xgap : p\ygap=ygap
	id=1
	
	; okay let's collect each of the image cells we need.
	cy=(ImageheightBB(IMtemp)-dhi)/2
	For y=1 To height-1
		cx=(ImagewidthBB(IMtemp)-dwid)/2
		For x=1 To width-1
			p\im[id]=CreateImageBB(cellsize,cellsize,0,True)
			CopyRectBB(cx,cy,cellsize,cellsize,0,0,IMtemp,p\im[id])
			cx=cx+cellsize
			id=id+1
		Next
		cy=cy+cellsize
	Next	
	p\totalim=id-1
	
	id=1 : ang#=inang
	For y=1 To height
		For x=1 To width
			; set up a point to rotate from
			p\point.flagbit[id]=New flagbit
			; and give it the current angle
			p\point[id]\ang=ang

			; figure out radius, you can get some wacky effects by messing with this.
			Select flagpole	
			Case FLAGPOLEtop
				p\point[id]\rad=Float(y+1)/Float(height)*rad	
			Case FLAGPOLEleft
				p\point[id]\rad=Float(x+1)/Float(width)*rad
			Case FLAGPOLEbottom
				p\point[id]\rad=(1-(Float(y-1)/Float(height)))*rad
			Case FLAGpoleright
				p\point[id]\rad=(1-(Float(x-1)/Float(width)))*rad	
			Default
				p\point[id]\rad=rad
			End Select
			ang=Wrap(ang+colchange,0,360)
			id=id+1
		Next
		ang=Wrap(inang+(rowchange*(y-1)),0,360)
	Next
	
	Return p
End Function



; This will update the flag's movement but won't draw it.
;   spd: is speed as if you couldn't guess. Negative values are also fine.
Function UpdateFlag(p.flagdefinition,spd#)
	If p=Null Then Return
	
	t=p\width*p\height
	For n=1 To t
		p\point[n]\ang=Wrap(p\point[n]\ang+spd,0,360)
	Next
End Function



; Call alpha -50 to have a cheap and cheerful lighting effect across the flag
;		alphamul: used when alpha is set to -50 to determine the strength of the darkening on the flag
Function RenderFlag(p.flagdefinition,x,y,xscale#=100,yscale#=100,alpha=255,alphamul=25)
	If p=Null Then Return

	; figure out change in x and y based on scale
	xch#=p\xgap*xscale
	ych#=p\ygap*yscale

	; right make a pointer for how far along/down the flag we've drawn
	cx#=x : cy#=y
	r=1 : c=1	; rows columns
	im=1		; which cell image we've reached
	
	points=p\width*(p\height-1)
	
	For n=1 To points
		; are we onto the last column yet?
		If c<p\width
			; now figure out and rotate each point for the cell/quad to draw
			; tl
			x1#=cx+cosine(p\point[n]\ang)*(p\point[n]\rad*xscale)
			y1#=cy+Sine(p\point[n]\ang)*(p\point[n]\rad*yscale)
			; tr
			x2#=cx+xch+Cosine(p\point[n+1]\ang)*(p\point[n+1]\rad*xscale)
			y2#=cy+Sine(p\point[n+1]\ang)*(p\point[n+1]\rad*yscale)
			; br
			x3#=cx+xch+Cosine(p\point[n+p\width+1]\ang)*(p\point[n+p\width+1]\rad*xscale)
			y3#=cy+ych+Sine(p\point[n+p\width+1]\ang)*(p\point[n+p\width+1]\rad*yscale)
			; bl
			x4#=cx+Cosine(p\point[n+p\width]\ang)*(p\point[n+p\width]\rad*xscale)
			y4#=cy+ych+Sine(p\point[n+p\width]\ang)*(p\point[n+p\width]\rad*yscale)
			
			al=alpha
			; darkening flag effect
			If alpha=-50
				; figure out if this point is lower than the next
				oy1#=cy+Sine(p\point[n]\ang)*(p\point[n]\rad)	
				oy2#=cy+Sine(p\point[n+1]\ang)*(p\point[n+1]\rad)
				; apply relevant alpha. Feel free to tweak the multiplier if it looks wrong.
				al=254+(oy1-(oy2))*alphamul
				If al<100 Then al=100
				If al>255 Then al=255
			End If
			Draw3DquadBB(p\im[im],x1,y1,x2,y2,x3,y3,x4,y4,al) : im=im+1
			cx=cx+xch
		Else
			; back to the first column, and onto the next row.
			c=0 : r=r+1 : cy=cy+ych : cx=x
		EndIf
		; okay along to the next column
		c=c+1
	Next
End Function

; copy an image into the flag using the current parameters.
;   Too slow for realtime animation I'm afraid, but much faster than creating a new flag from nothing.
Function CopyImageToFlag(p.flagdefinition,IMtemp)
	If p=Null Then Return
	
	; okay let's use the closest size we could find.
	dwid=p\cellsize*(p\width-1)
	dhi=p\cellsize*(p\height-1)
	
	; okay let's collect each of the image cells we need.
	id=1 : cy=(ImageheightBB(IMtemp)-dhi)/2
	For y=1 To p\height-1
		cx=(ImagewidthBB(IMtemp)-dwid)/2
		For x=1 To p\width-1
			CopyRectBB(cx,cy,p\cellsize,p\cellsize,0,0,IMtemp,p\im[id])
			cx=cx+p\cellsize
			id=id+1
		Next
		cy=cy+p\cellsize
	Next	
End Function

; Delete a flag
Function FreeFlag(p.flagdefinition)
	If p=Null Then Return
	; free up all the images
	For n=0 To p\totalim
		If p\im[n]<>False
			FreeImageBB(p\im[n]) : p\im[n]=0
		EndIf
	Next
	
	; and free the memory taken by the points.
	points=p\width*(p\height-1)
	For n=0 To points
		Delete p\point[n]
	Next
	
	Delete p
End Function

; Wrap variables
Function Wrap#(var#,lo#,hi#) 
	Local a#=Floor(var/(hi-lo)) 
	Return var-a*(hi-lo)+lo
End Function


; ------------------
; Okay let's test it out
;   Feel free To copy the above functions into your own game and discard everything below here.

; Guess my favourite amiga game. 
SetImageLoaderBB(1) 
IMflag=LoadImageBB("Rainbow.png")

; make a variable to hold a flag in
Local flag.flagdefinition


; Make a flag. This height doesn't quite fit in the y dimension but is a good match
;	this should run perfectly on anything with a geforce 2 and above
flag=CreateFlag(IMflag,20,12,16,16,100,22,16,10,FLAGPOLEleft)

; or try me instead if you have a great PC
;flag=CreateFlag(IMflag,40,25,8,8,100,22,18,12,FLAGPOLEleft)

; or you don't even have to worry about the parameters if you don't want to
;flag=CreateFlag(IMflag,20,12)

; don't need this anymore, createflag has already copied what it needs.

FreeImageBB(IMflag)


; make it big!
scale#=1.75
; move it on screen
MoveMouseBB(40,60)

Repeat
	FlipBB() : ClsBB()

	x=MouseXBB() : y=MouseYBB()
		
	UpdateFlag(flag,-4)
	RenderFlag(flag,x,y,scale,scale,-50,25)
	
	; clicky for resizing goodness.
	If MouseDownBB(1) Then scale=scale+0.01
	If MouseDownBB(2) Then scale=scale-0.01
	
	If scale<0.01 Then scale=0.01
	If scale>3 Then scale=3
Until KeyDown(1)

FreeFlag(flag)

EndGraphicsBB()
End