;=============
;Draw 3D Image
;=============
;This demo illustrates drawing Direct 3D-enabled sprites on screen.
;These sprites can utlize the real-time rotation, scaling, and alpha-
;blending abilities available in 3D hardware. If you don't need those 
;capabilities, however, normal images are a better choice (see below).

;Some of the limits of 3D images include:

;(a) 3D-enabled images are not supported in 8 bit (paletted)
;or 24 bit display modes.

;(b) For maximum video card compatibility, 3D-enabled images
;should have dimensions that are a power of 2. Example dimensions
;that work include 16x16, 32x32, 64x64, 128x128, and 256x256.
;Some cards may have difficulty with images larger than 256x256.
;If you fail to follow the power of 2 rule, your images may not 
;load properly, or at all.
;     Some cards may have difficulty with images that are not
;square (i.e. 32x64), though this may be quite rare. Also, 
;non-square images will look weird if you rotate them. If you 
;really need to rotate a non-square image, you must do it 
;"manually" using Draw3DQuadBB().

;(c) While 3D images are meant to be used in real time, they still 
;draw a bit more slowly than normal 2D images. Depending on how 
;many you are using and the hardware capabilities of the end-user, 
;this may or may not make a difference. In general, it is
;best to use purely 2D images when you don't need the added
;special effects capabilities of 3D-enabled images.

;(d) You can't use ReadPixelBB, WritePixelBB, ReadPixelFastBB, 
;WritePixelFastBB, or PlotBB on a 3D-enabled image. If you really
;want to do this, create a 2D image, write on that, and then use
;DrawImage to copy it over to the 3D image. See the Copy 2D to 3D
;demo for examples.

;(e) The following 2D commands also don't work with 3D images,
;though this isn't much of a sacrifice given their inherent 
;abilities: FlipImageBB, ResizeImageBB, RotateImageBB, ScaleImageBB

;(f) The two 3D drawing commands, Draw3DImage and Draw3DQuad, only
;work when drawing on the backbuffer. See the Copy 2D to 3D demo
;for notes on how to get around this.


Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16) ;8-bit and 24-bit not supported

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)

AutoMidHandleBB(True)
mage = LoadImageBB("graphics/mage2.bmp") ;normal 2D image
MaskImageBB(mage,71,71,71)

mage3D = Load3DImageBB("graphics/mage2.bmp") ;3D image
MaskImageBB(mage3D,71,71,71)

;Create a 3D-enabled image of a blue box
created3Dimage = CreateImageBB(32,32,0,1)
SetBufferBB(created3Dimage)
ColorBB(0,0,255)
RectBB(0,0,32,32,1)
ColorBB(255,255,255)
SetBufferBB(BackBufferBB)

alpha = 127 ;start as 50% alpha on 0-255 scale

;Main loop
While Not KeyDown(1)
	ClsBB()

	;Show frames per second
	Gosub fpsCounter
	TextBB(0,0,"FPS = " + FPS)	
	TextBB(0,550,"AvailSysMem (bytes) = " + AvailSysMemBB())		
	TextBB(0,570,"TotalSysMem (bytes) = " + TotalSysMemBB())			
	TextBB(550,550,"AvailVidMem (bytes) = " + AvailVidMem())		
	TextBB(550,570,"TotalVidMem (bytes) = " + TotalVidMem())	

	;Draw normal 2D image
	DrawImageBB(mage,100,100)
		
	;Draw normal 3D-enabled image
	Draw3dImageBB(mage3D,250,100)		

	;Draw 3D-enabled image flipped in real time
	Draw3dImageBB(mage3D,400,100,255,100,100,0,True)	
	
	;Draw 3D-enabled image rotating in real time
	angle = angle + 10
	If angle = 360 Then angle = 0
	Draw3dImageBB(mage3D,550,100,255,100,100,angle)
	Draw3dImageBB(created3Dimage,650,100,255,100,100,angle) ;draw blue box		

	;Draw 3D-enabled image squished in real time
	;Note that the placement of the image handle significantly effects
	;how this effect works. Try moving the handle around and check out
	;the difference.
	HandleImageBB(mage3D,64,128);put handle at bottom middle
	If squishVertical = True	
		Draw3dImageBB(mage3D,100,350,255,100,scale/2)
	Else	
		Draw3dImageBB(mage3D,100,350,255,scale/2,100)
	End If	
	MidHandleBB(mage3D)	;put handle back at middle of image	
				
	;Draw 3D-enabled image scaled in real time
	;See note above on handle placement.
	If scaleAscending = True
		scale = scale+3
		If scale >= 200 Then scaleAscending = False	
	Else
		scale = scale-3
		If scale <= 0 Then scaleAscending = True : squishVertical = 1 - squishVertical	
	End If	
	Draw3dImageBB(mage3D,300,300,255,scale,scale)	

	;Draw 3D-enabled image alpha-blended (fading in and out) 
	;Note that a -100 alpha will instead do an additive blend
	If alphaAscending = True
		alpha = alpha+1
		If alpha = 255 Then alphaAscending = False	
	Else
		alpha = alpha-1
		If alpha = 0 Then alphaAscending = True	
	End If	
	Draw3dImageBB(mage3D,MouseCursorX,MouseCursorY,alpha)

	;The works! Alpha-blended, scaled, rotated, and flipped!
	Draw3dImageBB(mage3D,600,300,alpha,scale,scale,angle,True)			

	;Optional FreeImage
;	If MouseDownBB(1) Then FreeImageBB(mage)
;	If MouseDownBB(2) Then FreeImageBB(mage3D)	

	;Draw Mouse
	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)
	FlipBB()
Wend

EndGraphicsBB()
End


.fpsCounter
	loops = loops+1	
	If startFPStimer = False Then 
		startFPStime = MilliSecs() + 1000
		startLoops = loops
		startFPStimer = True		
	Else If MilliSecs() > startFPStime Then
		startFPStimer = False
		FPS = loops-startLoops	
	End If
	Return