;=============
;Draw 3D Quad
;=============
;This demo illustrates the Draw3DQuad command. This command is basically
;a raw version of the Draw3DImage command where you specify the corner
;coordinates yourself. This command can do things that the Draw3DImage
;command can't, but you need to figure out where to draw the corners
;based on your needs.

;This demo clearly shows that a 3D image is really just a 2D image
;pasted onto two triangles, which are then scaled according to the 
;location of the corner coordinates.

;See the Draw3DImage demo for more notes on how to use 3D-enabled images.
;See the Quad Flag demo in the Demo Contributions folder to see
;what this command is truly capable of.


Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16) ;8-bit and 24-bit not supported in 3D

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)
mage3D = Load3DImageBB("graphics/mage2.bmp") ;3D image (unmasked, but you can mask it)

;Initialize the corner coordinates
Dim x(4), y(4)
Dim speed (8)
For z = 0 To 7
	speed(z) = 5-Rand(10)
Next
For z = 0 To 3
	x(z) = Rand(800) : y(z) = Rand(600)
Next

;This optional command prevents the drawing of 3D images that are
;flipped and therefore facing away from the front of the screen.
;Try uncommenting this line to see what it does.
;Cull3DImagesBB(True)

;Optional bilinear filtering, which makes scaled 3D images
;look less blocky, but more fuzzy.
;Filter3DImageBB(mage3D,True)

;Optional tint color for a 3D image. Default is white, which
;is 255,255,255.
;Tint3DImageBB(mage3D,0,0,255)

;Main loop
While Not KeyDown(1)
	ClsBB()
	
	;Show frames per second
	Gosub fpsCounter
	TextBB(0,0,"FPS = " + FPS)		
	
	;Move the corner coordinates around
    x(0) = x(0) + speed(0)
	y(0) = y(0) + speed(1)
    x(1) = x(1) + speed(2)
	y(1) = y(1) + speed(3)
    x(2) = x(2) + speed(4)
	y(2) = y(2) + speed(5)
    x(3) = x(3) + speed(6)
	y(3) = y(3) + speed(7)
	For z = 0 To 3
		If (x(z) < 0 Or x(z) > 800) Then speed(z*2) = -speed(z*2)
		If (y(z) < 0 Or y(z) > 600) Then speed(z*2+1) = -speed(z*2+1)		
	Next		

	;Draw the 3D quad (really it's two triangles)
	Draw3DQuadBB(mage3D,x(0),y(0),x(1),y(1),x(2),y(2),x(3),y(3))
			
	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)
	
	;Take a screen shot if you want.
	If MouseDownBB(1) Then SaveBufferBB(BackBufferBB,"myScreenShot.bmp")	
	FlipBB()
Wend

EndGraphicsBB()
End


.fpsCounter
	loops = loops+1	
	If startFPStimer = False Then 
		startFPStime = MilliSecs() + 1000
		startLoops = loops
		startFPStimer = True		
	Else If MilliSecs() > startFPStime Then
		startFPStimer = False
		FPS = loops-startLoops	
	End If
	Return