;===========================
;ImagesOverlap/ImagesCollide
;===========================
;Example of image overlap detection (which ignores transparency) and pixel perfect 
;collision detection (which considers transparency). Overlap detection is faster.

;Pixel-perfect collision detection is itself faster if at least the first of the two
;images is stored in RAM, rather than VRAM. Since VRAM images draw much
;faster on screen, if you do lots of collision detection, it might be worth loading
;two versions of an image, one in RAM and one in VRAM. Use the VRAM
;version to draw on screen. Use the RAM version for collision detection checks.

Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16)

;Load graphics
AutoMidHandleBB(True)
AutoMaskImageBB(True,71,71,71)
mage = LoadImageBB("graphics/mage2.bmp",1) ;faster if first image is in RAM
mage2 = LoadImageBB("graphics/mage2.bmp")

MoveMouseBB(200,200)

;Main loop
While Not KeyDown(1)
	ClsBB()

	DrawImageBB(mage,400,300)
	DrawImageBB(mage2,MouseCursorX,MouseCursorY)

	z1 = ImagesOverlapBB(mage,400,300,mage2,MouseCursorX,MouseCursorY)	
	z2 = ImagesCollideBB(mage,400,300,0,mage2,MouseCursorX,MouseCursorY,0)
	
	TextBB(0,0,"ImagesOverlapBB = " + z1)
	TextBB(0,20,"ImagesCollideBB = " + z2)		
	
	FlipBB()
Wend

DebugLog EndGraphicsBB()
End