;========
;Palettes
;========
;Shows how to manipulate palettes when in 8 bit color depth mode.
;Also, check out the notes in the Readme.txt file.

Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,8,"Palettes","graphics/mage2.bmp")
SetFontBB("Courier",14)

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)
mage = LoadImageBB("graphics/mage.bmp");load to VRAM
paletteImage = CreatePaletteImage()

;Find the palette entry that is closest to a specific rgb value
r = 71 : g = 71 : b = 71
closestPalette = ColorMatchBB(r,g,b)


;Main loop
While Not KeyDown(1)
	ClsBB()

	TextBB(0,0,"Current palette entry under mouse tip = " + paletteEntry)
	TextBB(0,20,"Red component = " + ReadPaletteR_BB(paletteEntry))	
	TextBB(0,40,"Green component = " + ReadPaletteG_BB(paletteEntry))
	TextBB(0,60,"Blue component = " + ReadPaletteB_BB(paletteEntry))
	TextBB(50,520,"Press 1 to change 1 palette entry")	
	TextBB(50,540,"Press 2 to change a range of palette entries")	
	TextBB(50,560,"Press 3 to change all palette entries")					
	TextBB(490,70,"The Current Palette")
	TextBB(400,450,"The closest palette entry to "+r+","+g+","+b+" = " + closestPalette)	

	;Read palette entry under mouse and r,g,b values of that palette entry	
	DrawImageBB(mage,150,200)
	DrawBlockBB(paletteImage,400,100)	
	paletteEntry = ReadPixelBB(MouseCursorX,MouseCursorY)					

	;Change palette entries by pressing 1, 2 or 3.
	If KeyDown(2)	 ;change a single palette entry
		WritePaletteBB(223,0,255,255)	
		SetPaletteBB(223,1)		
	Else If KeyDown(3)	;change a range of palette entries
		For x = 220 To 255
			WritePaletteBB(x,255,0,255)	
		Next				
		SetPaletteBB(220,36)			
	Else If KeyDown(4)	;change every item in palette
		For x = 0 To 255
			WritePaletteBB(x,255,255,0)	
		Next	
		SetPaletteBB()					
	End If
	
	;Draw mouse
	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)	
	
	;Take a screen shot if you want.
	If MouseDownBB(1) Then SaveImageBB(paletteImage,"myPalette.bmp")
	FlipBB()
Wend

EndGraphicsBB()
End


;Creates a 16x16 block image to show the current palette
Function CreatePaletteImage()
	paletteImage = CreateImageBB(320,320)
	LockBufferWritingBB(paletteImage,320*320)	
	For paletteEntry = 0 To 255
		x = paletteEntry Mod 16	: y = paletteEntry/16
		For a = x*20 To x*20+19
		For b = y*20 To y*20+19
			WritePixelFastBB(a,b,paletteEntry)
		Next
		Next	
	Next
	UnlockBufferWritingBB(paletteImage)	
	Return paletteImage
End Function