;==========
;Read Pixel
;==========

;This demo will report the r,g, and b values of the pixel under the mouse.

;The "naked pixel" data isn't anything you really need to pay attention
;to, but might be interesting to folks who want to know what is
;happening behing the scenes. This is what is actually written in memory.
;As the color depth increases from 8 to 16 to 24/32, the amount of
;memory consumed by the pixel increases. Try changing the color depth
;in the GraphicsBB() command and check out how much data there
;is for each pixel. This is why lower color depths consume less VRAM.

;The rgb data represents a conversion of the naked pixel data to the
;usual r, g, and b color values you are used to.


Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16)
SetFontBB("Courier",14)
SetBufferBB(BackBufferBB)

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)
mage = LoadImageBB("graphics/mage.bmp") 


While Not KeyDown(1)
	ClsBB()
	
	DrawImageBB(mage,25,200)		

	TextBB(200,0,"Current ColorDepth = " + ColorDepth + " bits per pixel (8 bits = 1 byte)")	
	TextBB(200,20,"Current MouseX = " + MouseCursorX)
	TextBB(200,40,"Current MouseY = " + MouseCursorY)	
	
	;Read the pixel data	
	If ColorDepth = 8
		paletteEntry = ReadPixelBB(MouseCursorX,MouseCursorY)			
		rgb = ReadPaletteRGB_BB(paletteEntry)	
	Else
		rgb = ReadPixelBB(MouseCursorX,MouseCursorY)			
	End If		

	;Report naked pixel info	
	If ColorDepth = 8 nakedPixel = paletteEntry	
	If ColorDepth = 16 nakedPixel = PeekShort(BankStartOut,0)
	If ColorDepth = 24 nakedPixel = PeekInt(BankStartOut,0)		
	If ColorDepth = 32 nakedPixel = PeekInt(BankStartOut,0)				
	TextBB(200,80,"Naked Pixel = " + nakedPixel)							
	TextBB(200,100,"Binary Equivalent   = " + Bin$(nakedPixel))		
	TextBB(200,120,"Binary Byte Marker  = X0000000X0000000X0000000X0000000")	

	;Report RGB data			
	If ColorDepth = 8 ;if in 8 bit mode, read pixel gives you a palette entry		
		TextBB(200,180,"Palette Entry = " + paletteEntry)			
	Else	
		TextBB(200,180,"RGB = " + rgb)
	End If				
	TextBB(200,200,"- r = " + GetR_BB(rgb))
	TextBB(200,220,"- g = " + GetG_BB(rgb))
	TextBB(200,240,"- b = " + GetB_BB(rgb))	
	
	TextBB(200,320,"AvailSysMem (bytes) = " + AvailSysMemBB())		
	TextBB(200,340,"TotalSysMem (bytes) = " + TotalSysMemBB())			
	TextBB(200,360,"AvailVidMem (bytes) = " + AvailVidMem())		
	TextBB(200,380,"TotalVidMem (bytes) = " + TotalVidMem())	
	
	If ColorDepth = 16 TextBB(200,420,"screenFormat16Bit = " + screenFormat16Bit)	

	DrawImageBB(mousePointer,MouseCursorX,MouseCursorY)
	FlipBB()
Wend

EndGraphicsBB()
End