;================
;Userlib Commands
;================

;This demo will only work in B+ and B3D, and only if you copy
;the ExtendedB2D.decls file and the Blitz2DDll.dll to your userlib 
;folder.

;Userlibs are a bit faster than the Blitz CallDLL command, which is
;what the Extended B2D library mostly relies on. The reason why it
;uses CallDLL is because Blitz 2D can not use userlibs. They are only
;usable in B+ and B3D.

;In most cases, the speed difference is not noticeable. You need to
;use a command dozens or hundreds of times per frame to pick up
;even a few milliseconds in speed. Nevertheless, there are a few
;commands where this is a worthwhile speed increase - notably
;the core image drawing commands.

;In this version of the Extended B2D library, just 4 commands have 
;userlib equivalents. They are:

;- Draw3DQuadUL
;- Draw3DImageUL
;- DrawBlockUL
;- DrawImageUL

;In each case, this command is identical to its normal counterpart
;except that:

; - It it has a "UL" at the end instead of a "BB" at the end
; - You need to specify ALL of the parameters. There are no optional
;   parameters with userlibs (for example, when you use the DrawImageUL
;   command, you must specify the frame, which is 0 for non-animated
;   images).

;Also, please note that the dllFileName$ command/feature (see any of
;the other demos) should be avoided if you use userlib commands.
;The Blitz2DDll.dll should be in your userlibs folder during
;development, and in the same folder as your main game program
;when you generate your final exe.


Include "../ExtendedBlitzLib.bb"
;dllFileName$ = ....  -> don't use this if you use userlibs commands.
GraphicsBB(800,600,16)

;Load graphics
mousePointer = LoadImageBB("graphics/red pointer.bmp")
MaskImageBB(mousePointer,255,255,255)
AutoMidHandleBB(True)
mage = LoadImageBB("graphics/mage.bmp")
MaskImageBB(mage,71,71,71)

mage3D = Load3DImageBB("graphics/mage.bmp") ;3D image
MaskImageBB(mage3D,71,71,71)

;Main loop
While Not KeyDown(1)
	ClsBB()

	;Show frames per second
	Gosub fpsCounter
	TextBB(0,0,"FPS = " + FPS)

	;Draw normal 2D images using userlib commands
	DrawBlockUL(mage,300,100,0)	
	DrawImageUL(mage,500,100,0)

	;Draw 3D-enabled images using userlib commands
	Draw3DImageUL(mage3D,MouseCursorX,MouseCursorY,255,100,100,45,False)	
	Draw3DQuadUL(mage3D,200,200,400,200,400,400,200,400,255)
	
	;Draw Mouse using userlib DrawImageUL
	DrawImageUL(mousePointer,MouseCursorX,MouseCursorY,0)

	;Take a screen shot if you want.
	If MouseDownBB(1) Then SaveBufferBB(BackBufferBB,"myScreenShot.bmp")
	FlipBB()
Wend

EndGraphicsBB()
End



.fpsCounter
	loops = loops+1	
	If startFPStimer = False Then 
		startFPStime = MilliSecs() + 1000
		startLoops = loops
		startFPStimer = True		
	Else If MilliSecs() > startFPStime Then
			startFPStimer = False
			FPS = loops-startLoops	
	End If
	Return