;===============
;WritePixelsFast
;===============
;This demo shows how to use the ReadPixelFastBB and WritePixelFast
;functions.

;These commands are no faster than the native Blitz commands, so don't
;expect much. In general, they will be faster the lower your color depth is.
;256 color/8 bit will be the fastest.

;Please Note: Using WritePixelFastBB on an image > 200x200 pixels requires
;adding the size or you will get an error. See the comments in the 
;ExtendedBlitzLib for more info.


Include "../ExtendedBlitzLib.bb"
dllFileName$ = "../Blitz2DDll.dll" ;optional (if file not in same directory)
GraphicsBB(800,600,16)
SetBufferBB(BackBufferBB)

;Load graphics
mage = LoadImageBB("graphics/mage2.bmp");load to VRAM
MaskImageBB(mage,71,71,71)
mage2 = LoadImageBB("graphics/mage2.bmp",1);load to RAM (no reason, just to show how)
MaskImageBB(mage2,71,71,71)

;Write pixels from top half of one image to bottom
;half of another.
LockBufferReadingBB(mage2,0,0,128,64)  ;read part of image	
LockBufferWritingBB(mage)
For x = 0 To 127;one less than image width since starting at 0
For y = 0 To 63 ;one less than height of read image area since starting at 0
	If ColorDepth = 8 
		WritePixelFastBB(x,y+64,ReadPixelFastBB(x,y))
	Else
		rgb = ReadPixelFastBB(x,y)
		WritePixelFastBB(x,y+64,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
	End If	
Next
Next	
UnlockBufferWritingBB(mage)	

;Main loop
While Not KeyDown(1)
	ClsBB()

	DrawImageBB(mage,200,100)		
	DrawImageBB(mage2,200,300)		

	;Write some random pixels in a box 
	LockBufferWritingBB(BackBufferBB)	
	For x = 0 To 100
	For y = 0 To 100	
		WritePixelFastBB(x+400,y+300,Rand(255),Rand(255),Rand(255))	
	Next
	Next
	UnlockBufferWritingBB(BackBufferBB)	

	;Read some pixels from an image and write them to the 
	;backbuffer (not recommended beyond this demo
	;because it's slow).
	LockBufferReadingBB(mage2) ;this copies whole image
	LockBufferWritingBB(BackBufferBB)
	For x = 0 To 127 ;one less than image width since starting at 0
	For y = 0 To 127 ;one less than image height since starting at 0
		If ColorDepth = 8 
			WritePixelFastBB(x+400,y+100,ReadPixelFastBB(x,y))		
		Else
			rgb = ReadPixelFastBB(x,y)
			WritePixelFastBB(x+400,y+100,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
		End If	
	Next
	Next	
	UnlockBufferWritingBB(BackBufferBB)
		
	;Read some pixels from the backbuffer and write them somewhere
	;else on the backbuffer (again, not that fast)
	LockBufferReadingBB(BackBufferBB,200,300,128,64)  ;copy part of backbuffer	
	LockBufferWritingBB(BackBufferBB)	
	For x = 0 To 127 ;one less than width since starting at 0
	For y = 0 To 63 ;one less than height since starting at 0
		If ColorDepth = 8 
			WritePixelFastBB(x+600,y+300,ReadPixelFastBB(x,y))		
		Else
			rgb = ReadPixelFastBB(x,y)
			WritePixelFastBB(x+600,y+300,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
		End If			
	Next
	Next
	UnlockBufferWritingBB(BackBufferBB)		

	;Take a screen shot if you want.
	If MouseDownBB(1) Then SaveBufferBB(BackBufferBB,"myScreenShot.bmp")
	FlipBB()
Wend

EndGraphicsBB()
End