;=========================
;EXTENDED BLITZ 2D v. 1.60
;=========================
;By Patrick Lester (aka turtle1776), pwlester@policyalmanac.org
;- Updated 12/1/03

;This collection of functions, along with the corresponding dll, provide 
;expanded capability that was not built into the original version of 
;Blitz 2D. These functions replace their corresponding native Blitz 2D
;counterparts. See the Blitz help files for further details, along with
;the readme.txt file for the few commands that are purely Blitz 2D 
;Extended.

;Just like Blitz, these functions require the user to have DX 7 or higher
;installed. Since we are now up to DirectX 9, and DirectX 7 is a few years
;old, this should be no problem. If the user doesn't have DX 7 installed,
;the game will quit and flash a message box telling them to upgrade.


;Major Capabilities
;------------------
;- Real-time rotations, scaling, and alpha-blending (see Draw3DImage demo)
;- 8 bit graphics mode, including palette reading/manipulation
;- load images to, and draw them from, RAM instead of VRAM
;- extended GUI drawing (dotted lines, fill patterns)
;- read available and total system memory (RAM)

;- C++ dll source code provided for further expanded capability
;  for those who know, or are willing to learn, C++ and DirectX.


;==========================================================
; STANDARD LEGAL MUMBO-JUMBO TO COVER MY BUTT 
; -------------------------------------------
; CODE PROVIDED UNDER THIS LICENSE IS ON AN "AS IS" BASIS, WITHOUT WARRANTY
; OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
; THAT THE COVERED CODE IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE
; OR NON-INFRINGING. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE COVERED
; CODE IS WITH YOU. SHOULD ANY COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT
; THE INITIAL DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
; SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL
; PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
; THIS DISCLAIMER.
; 
; In other words, while I have no reason to think there are any problems
; with this dll and Blitz code, don't sue me if something goes wrong. Use at 
; your own risk. Blah, blah, blah.
;==========================================================

	
;==============
;GLOBAL VARIABLES

Global dllFileName$ = "Blitz2DDll.dll"
Global BankStartIn  = CreateBank(1024) ;Bank to go into dll function
Global BankStartOut 	= CreateBank(1024) ;Bank of info from dll function
Global ScreenWidth, ScreenHeight, ColorDepth, ScreenFormat16Bit
Const FrontBufferBB = 1, BackBufferBB = 2
Global ReadPixelPitch, currentFontBB = 0
Global pixelReadBank = CreateBank(1) ;bank used to read pixels from for ReadPixelFast
Global pixelWriteBank = CreateBank(1) ;bank used to write pixels to for WritePixelFast
Global MouseCursorX, MouseCursorY, LeftMouseButton, RightMouseButton
Global MouseHit1BB, MouseHit2BB, WinMouseCursorX, WinMouseCursorY
Global BlitzPlusVersion = False, BlitzPlusWindowHandle
Global gImageLoader=0

;==============
;FUNCTIONS

;This function changes the alpha blending for a 3D image by
;adjusting the alpha associated with each of its four
;corner vertices. This allows variable blending across
;the face of the image, rather than using a uniform alpha
;blend for the entire image. The first vertex in the list
;of parameters is the upper left corner of the image. The
;other vertices proceed in clockwise order from there.
;Vertex alpha will over-ride the alpha parameter in Draw3DImageBB
;and Draw3DQuad. To turn off alpha vertex, set it to 255,255,255.
;- Example = Alpha Effects.bb demo
Function AlphaVertexBB(imagePointer,vertex1,vertex2,vertex3,vertex4)
	PokeInt BankStartIn,0,imagePointer
	PokeInt BankStartIn,4,vertex1	
	PokeInt BankStartIn,8,vertex2	
	PokeInt BankStartIn,12,vertex3	
	PokeInt BankStartIn,16,vertex4					
	Return CallDLL(dllFileName$,"AlphaVertexBB",BankStartIn)	
End Function

;Automatically masks all images loaded or created after the
;command is called using the selected color. Turns off 
;AutoMaskImagePixelBB if it is on.
;Value = true turns it on, false turns it off.
;- Example = ImagesCollide.bb
Function AutoMaskImageBB(value=True,r=0,g=0,b=0)
	PokeInt BankStartIn,0,value	
	PokeInt BankStartIn,4,r	
	PokeInt BankStartIn,8,g	
	PokeInt BankStartIn,12,b				
	CallDLL(dllFileName$,"AutoMaskImageBB",BankStartIn)	 
End Function

;Automatically masks all images loaded or created after the
;command is called using the color at the selected pixel location.
;Always masks images created with CreateImageBB to the color black 
;because that is their color at the time they are created. If
;you want a different color you will need to remask it.
;Turns off AutoMaskImageBB if it is on.
;Value = true turns it on, false turns it off.
Function AutoMaskImagePixelBB(value=True,x=0,y=0)
	PokeInt BankStartIn,0,value	
	PokeInt BankStartIn,4,x	
	PokeInt BankStartIn,8,y					
	CallDLL(dllFileName$,"AutoMaskImagePixelBB",BankStartIn)	 
End Function

;Automatically sets the image handle of loaded images to the middle of 
;the image. Only affects images loaded or created after the command
;has been used. 
;Value = true turns it on. Value = false turns it off.
;- Example = DrawImage.bb
Function AutoMidHandleBB(value)
	PokeInt BankStartIn,0,value	
	CallDLL(dllFileName$,"AutoMidHandleBB",BankStartIn)	 
End Function

;Get available physical system memory in bytes. This function may 
;give inaccurate numbers, and does not provide virtual system memory
;created on hard disk. It is good for tracking potential memory leaks
;if you decide to edit or update the dll.
;- Example = Memory Leak demo
Function AvailSysMemBB()
	Return CallDLL(dllFileName$,"AvailSysMemBB")	
End Function	

;Returns a "pointer" to the back buffer.
Function BackBufferBB()
	Return BackBufferBB	
End Function		

;Clears the current drawing buffer with the designated CLS color, 
;which is black unless changed by the ClsColorBB() command. Default
;usage, which involves just using ClsBB(), clears the entire buffer.
;You can also just clear a portion of the buffer by designated the
;x and y values of the upper left hand corner, and the width and
;height of the area being cleared. If the area cleared falls outside
;of the dimensions of the buffer, the command will fail.
;- Example = DrawImage.bb demo
Function ClsBB(x=0,y=0,width=-1,height=-1)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y  
	PokeInt BankStartIn,8,width  	
	PokeInt BankStartIn,12,height  	
	result = CallDLL(dllFileName$,"ClsBB",BankStartIn)		
End Function		

;Changes the color for the Cls command. Default is black.
Function ClsColorBB(r,g,b)
	PokeInt BankStartIn,0,r  	
	PokeInt BankStartIn,4,g  
	PokeInt BankStartIn,8,b 	
	Return CallDLL(dllFileName$,"ClsColorBB",BankStartIn)	
End Function	

;Changes the current drawing color for text writing and GUI draw
;commands like LineBB, OvalBB, and RectBB. You can set each
;of these separately using SetFontColorBB, SetBrushBB and 
;SetPenBB. ;
;	In 8-bit mode this command is slow -- too slow for real-time
;use. This is because the command needs to search through the entire
;palette to find the closest palette entry. Unfortunately, the 
;underlying Microsoft Windows commands being used can not use
;palette entries, which would be a lot faster.
;-Example = GUI Commands.bb demo.
Function ColorBB(r,g,b)
	SetFontColorBB(r,g,b)
	SetBrushBB(r,g,b)
	SetPenBB(r,g,b)
End Function		

;Used To retrieve the closest pixel value To a given rgb. The
;returned pixel value will vary based upon the display mode that is
;being used. This command is primarily used in 8-bit mode when trying
;find the closest palette entry To a particular rgb value.
;- Example = Palettes.bb demo.
Function ColorMatchBB(r,g,b)
	PokeInt BankStartIn,0,r  	
	PokeInt BankStartIn,4,g  
	PokeInt BankStartIn,8,b 	
	Return CallDLL(dllFileName$,"ColorMatchBB",BankStartIn)
End Function

;Creates a new image which is a copy of an existing image.
;If the existing image is an animated image, the
;created image will have the same attributes.
;	When creating a 3D image from a 2D image, this command
;will probably fail if the width and length of the source
;image are not a power of 2 (i.e., 32x64). See the Draw3DImage.bb
;demo for more information about 3d-enabled images.
;  Also, when creating a 2D image from a 3D image in 16 bit
;display mode, the colors may be very slightly altered. This 
;change will probably not be noticeable, but it
;is enough to screw up masking. In such cases, it is 
;probably better to use MaskImagePixelBB or AutoMaskImagePixelBB.
;- loadTo = 0 loads to VRAM; loadTo = 1 loads to RAM.
;  This parameter is ignored when creating a 3D-enabled
;  image, though. 
;Example = Copy 2D to 3D.bb demo
Function CopyImageBB(imagePointer,loadTo=0,make3D=False)
	If loadTo > 1 Then Return
	If make3D=True And ColorDepth = 8 Then Return
	PokeInt BankStartIn,0,imagePointer	
	PokeInt BankStartIn,4,loadTo			
	PokeInt BankStartIn,8,make3D	
	Return CallDLL(dllFileName$,"CopyImageBB",BankStartIn)	
End Function	

;Copies a portion of one image onto another, or 
;onto the current drawing buffer if no image is 
;specified. This command ignores mask and handle
;information.
;- Example = CopyRect.bb demo
Function CopyRectBB(x1,y1,width,height,x2,y2,imagePointer1=-1,imagePointer2=-1)
	PokeInt BankStartIn,0,x1 	
	PokeInt BankStartIn,4,y1  		
	PokeInt BankStartIn,8,width 	
	PokeInt BankStartIn,12,height  	
	PokeInt BankStartIn,16,x2 	
	PokeInt BankStartIn,20,y2  	
	PokeInt BankStartIn,24,imagePointer1 	
	PokeInt BankStartIn,28,imagePointer2  		
	Return CallDLL(dllFileName$,"CopyRectBB",BankStartIn)	
End Function	

;Creates an alpha textured 3D image from two other component
;images. For more information, see the Alpha Images.bb demo.
;- Example = Alpha Images.bb demo.
Function CreateAlphaImageBB(image,alphaTexture)
	PokeInt BankStartIn,0,image 	
	PokeInt BankStartIn,4,alphaTexture  
	Return CallDLL(dllFileName$,"CreateAlphaImageBB",BankStartIn)
End Function	

;Creates a blank (black) image of the specified dimensions.
;loadTo = 0 loads to VRAM; loadTo = 1 loads to RAM.
;This parameter is ignored when creating a 3D-enabled
;image, though. Also, 3D-enabled images should have dimensions 
;that are a power of two. You can not create animated images 
;with this command.
;	When creating a 3D image, this command will probably fail 
;if the width (x) and length (y) are not a power of 2. 
;Example = Copy 2D to 3D demo
Function CreateImageBB(x,y,loadTo=0,make3D=False)
	If make3D=True And ColorDepth = 8 Then Return
	PokeInt BankStartIn,0,x 	
	PokeInt BankStartIn,4,y  
	PokeInt BankStartIn,8,loadTo	
	PokeInt BankStartIn,12,make3D	
	Return CallDLL(dllFileName$,"CreateImageBB",BankStartIn)	
End Function	

;Prevents a 3D image or portion of a 3D image from being drawn
;if it is facing away from the screen. Primarily used with the 
;Draw3DQuadBB command. The only affect on the Draw3DImage command is
;causing images drawn flipped to not be drawn at all.
;- Value = true turns culling on; false = turns it off (default)
;Example = Draw3DQuad.bb demo.
Function Cull3DImagesBB(value)
	PokeInt BankStartIn,0,value 
	Return CallDLL(dllFileName$,"Cull3DImagesBB",BankStartIn)		
End Function

;Draws a 3D-enabled image on screen. This command will not work
;unless the image has been loaded with Load3DImageBB or created
;as a 3D image with CreateImageBB or CopyImageBB.
;	Also, this command only supports drawing onto the backbuffer.
;SetBufferBB() is ignored. If for some reason you want to capture 
;an image drawn with Draw3DImage or Draw3DQuad, try clearing the
;backbuffer, drawing the image however you want, and then using
;SetBufferBB and CopyRectBB to draw it from the backbuffer to
;a 2D image.
; - alpha = 0-255, with 0 = invisible and 255 = opaque; -100 = additive alpha
;   This will be overridden by the AlphaVertex command if it has been
;   used and set to anything other than 255,255,255.
; - scale = any percentage > 0. 100% = normal size. Also, note that handle
;   placement is proportionally scaled when you use this command.
; - angle = 0-360 degrees in a counter-clockwise direction
; - flipped = true/false. Flips the image horizontally. May
;	  be combined with angle 180 for vertical flip.
;Example = Draw3DImage demo
Function Draw3DImageBB(imagePointer,x,y,alpha=255,xScale=100,yScale=100,angle=0,flipped=False)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,x  	
	PokeInt BankStartIn,8,y  		
	PokeInt BankStartIn,12,alpha	
	PokeInt BankStartIn,16,xScale	
	PokeInt BankStartIn,20,yScale		
	PokeInt BankStartIn,24,angle
	PokeInt BankStartIn,28,flipped		
	Return CallDLL(dllFileName$,"Draw3DImageBB",BankStartIn)	
End Function

;Similar To Draw3DImageBB, except that you specify the raw corner
;coordinates. Handle information is ignored with this command.
;x1/y1 = upper left hand coordinate of original image. The other
;3 coordinates are run in clockwise order from that point.
;- Example = Draw3DQuad demo
Function Draw3DQuadBB(imagePointer,x1,y1,x2,y2,x3,y3,x4,y4,alpha=255)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,x1  	
	PokeInt BankStartIn,8,y1  		
	PokeInt BankStartIn,12,x2	
	PokeInt BankStartIn,16,y2
	PokeInt BankStartIn,20,x3		
	PokeInt BankStartIn,24,y3
	PokeInt BankStartIn,28,x4		
	PokeInt BankStartIn,32,y4	
	PokeInt BankStartIn,36,alpha	
	Return CallDLL(dllFileName$,"Draw3DQuadBB",BankStartIn)	
End Function

;Draws an unmasked image on the screen.
;This command is slow for 3D-enabled images. Use 
;Draw3DImageBB instead.
;- Example = DrawImage demo
Function DrawBlockBB(imagePointer,x,y,frame=0)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,x  	
	PokeInt BankStartIn,8,y  
	PokeInt BankStartIn,12,frame
	Return CallDLL(dllFileName$,"DrawBlockBB",BankStartIn)	
End Function

;Draws a masked image on the screen
;This command is slow for 3D-enabled images. Use 
;Draw3DImageBB instead. Please note that if this
;command is used on a 3D image, either as the source
;or destination image, the image will be drawn without
;its mask. It will also be much slower than using Draw3DImage.
;- Example = DrawImage demo
Function DrawImageBB(imagePointer,x,y,frame=0)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,x  	
	PokeInt BankStartIn,8,y  
	PokeInt BankStartIn,12,frame	
	Return CallDLL(dllFileName$,"DrawImageBB",BankStartIn)	
End Function	

;Draws a portion of an image on the current drawing buffer.
;This command is very slow for 3D-enabled images.
;This command considers mask information, just like DrawImageBB.
;It ignores image handle information, however.
;- Example = CopyRect.bb demo
Function DrawImageRectBB(imagePointer,x,y,imageX,imageY,width,height)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,x  	
	PokeInt BankStartIn,8,y  
	PokeInt BankStartIn,12,imageX	
	PokeInt BankStartIn,16,imageY  	
	PokeInt BankStartIn,20,width  	
	PokeInt BankStartIn,24,height  	
	Return CallDLL(dllFileName$,"DrawImageRectBB",BankStartIn)	
End Function	

;Releases all used memory/VRAM and exit graphics mode. This command
;MUST be called at the end of your program, before the Blitz "End"
;command is used. Failing to do this will prevent you from relaunching
;the program until the Blitz IDE is closed and reopened.
;;- Example = DrawImage.bb demo (all demos, really).
Function EndGraphicsBB()
	ClsColorBB(0,0,0)
	ClsBB()
	FlipBB()
	xx = CallDLL(dllFileName$,"EndGraphicsBB")
	
	;Blitz Plus users, uncomment this line
;	FreeGadget BlitzPlusWindowHandle	
	Return xx
End Function

;Turns on bi-linear filtering for a 3D-enabled image. This will
;cause an image that is rendered at a different scale (larger
;or smaller) to be rendered with a filtering effect that makes
;it a bit smoother, less blocky, and a bit fuzzy. This effect
;slows down rendering a bit, so you should generally avoid it
;unless you need it for some reason.
;- Value = true turns filtering on for the image, false turns it off
;Example = Image Filtering.bb demo. 
Function Filter3DImageBB(imagePointer,value)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,value  
	Return CallDLL(dllFileName$,"Filter3DImageBB",BankStartIn)
End Function

;Flips the backbuffer to the frontbuffer
;- Example = DrawImage.bb demo
Function FlipBB()
	MouseBufferCheckBB() ;check buffered MouseHit info
	UpdateMouseBB() ;update current mouse info	
	Return CallDLL(dllFileName$,"FlipBB")
End Function

;Flips an image horizontally, vertically, or both. This change is permanent,
;unlike Draw3DImage, which can draws a flipped version of an image onscreen
;in real time, but does not change the image itself.
;    This command does not work with 3D-enabled images. Also, when used
;on an animated image it will flip the whole image, not each individual
;frame.
;- Mode 0 = horizontal mirror; 1 = vertical; 2 = both (180 degree rotate)
Function FlipImageBB(imagePointer,mode=0)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,mode  	
	Return CallDLL(dllFileName$,"FlipImageBB",BankStartIn)	
End Function

;Frees all images and the memory associated with them. This is
;done automatically when you call EndGraphicsBB(), so this
;command is not required. Unlike the FreeImageBB command, this
;command not only frees the image itself, but also the memory
;used to record other information about the image, like its mask color,
;handle information, whether it is a 3D image, etc. This command
;totally frees ALL memory associated with your existing images.
;	If you use this command, you must immediately reload all of your 
;images and/or set pre-existing image pointers to zero. If you don't 
;do this, and then use a DrawImageBB or similar drawing command on
;one of your pre-existing images, you will get an error and your 
;game will crash.
;- Example = Alt Tab.bb demo
Function FreeAllImagesBB()	
	CallDLL(dllFileName$,"FreeAllImagesBB")	
End Function

;Free an image that has been loaded or created.
;-Example = Draw3DImage.bb demo.
Function FreeImageBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer 	
	CallDLL(dllFileName$,"FreeImageBB",BankStartIn)	
End Function	

;Returns a "pointer" to the front buffer.
Function FrontBufferBB()
	Return FrontBufferBB	
End Function

;Sets the current drawing color to the color of the pixel designated.
;Does not work with 3D-enabled images.
Function GetColorBB(x,y)
	rgb = ReadPixelBB(x,y)
	r = GetR_BB(rgb)
	g = GetG_BB(rgb)
	b = GetB_BB(rgb)	
	ColorBB(r,g,b)
End Function

;Combines separate r,g and b values into a single RGB value.
Function GetRGB_BB(r,g,b)
	Return (r Shl 16) Or (g Shl 8) Or b 
End Function

;Returns r component of an RGB value.
Function GetR_BB(rgb)
    Return rgb Shr 16 And %11111111
End Function

;Returns g component of an RGB value.
Function GetG_BB(rgb)
	Return rgb Shr 8 And %11111111
End Function

;Returns b component of an RGB value.
Function GetB_BB(rgb)
	Return rgb And %11111111
End Function

;Returns the current mouse speed. Starting speed is 2.
;This speed does not match the Windows system mouse speed.
Function GetMouseSpeedBB()
	Return CallDLL(dllFileName$,"GetMouseSpeedBB")
End Function

;This is an internal debugging function. Please ignore.
Function GetValueBB(value)
	PokeInt BankStartIn,0,value
	Return CallDLL(dllFileName$,"GetValueBB",BankStartIn) 
End Function

;Returns the version number of the Blitz2DDll.dll.
;- Example = GetVersion.bb demo.
Function GetVersionBB#()
	version# = CallDLL(dllFileName$,"GetVersionBB") 
	Return version/100
End Function

;Launches Blitz in Extended Graphics mode. Windowed
;mode is unavailable.
;- Example = DrawImage.bb demo.
Function GraphicsBB(width=800,height=600,depth=16,programTitle$="Blitz Extended",paletteFile$="palette.bmp")

	;For Blitz Plus, uncomment these two lines AND the one in the
	;EndGraphicsBB command.
;	BlitzPlusWindowHandle = CreateWindow(programTitle$,0,0,300,300,0,0)
;	BlitzPlusVersion = True	

	AppTitle(programTitle$) ;used by dll's MessageBoxBB and GraphicsBB functions
	ScreenWidth = width : ScreenHeight = height 
	ColorDepth = depth ;record color depth for later use. This is a global.

	;Load the Blitz dll	
	result = CallDLL(dllFileName$,"loadDLL") ;original call loads the dll		
	If result <> 1
		RuntimeError "Unable to load Blitz2DDll.dll"
		End
	End If	
	
	;Launch DirectX
	PokeInt BankStartIn,0,width  	
	PokeInt BankStartIn,4,height  	
	PokeInt BankStartIn,8,ColorDepth  	
	result = StringToAsciiBB(programTitle$,BankStartIn,12)
	result = StringToAsciiBB(paletteFile$,BankStartIn,result)		
	result = CallDLL(dllFileName$,"GraphicsBB",BankStartIn)
	
	;Process errors, if any
	If result = 0
		MessageBoxBB(programTitle$,"Game Launch Error","Unable to launch game. Blitz Plus users, please see Readme.txt") 
	Else If result = 2
		MessageBoxBB(programTitle$,"Game Launch Error","Please upgrade your version of DirectX. It is available for free at http://www.microsoft.com/windows/directx.") 
	Else If result = 4
		MessageBoxBB(programTitle$,"Game Launch Error","Unable to start in specified graphics mode. Sorry.") 
	Else If result <> 1
		MessageBoxBB(programTitle$,"Game Launch Error","Unable to launch game. Error " + result + ". Sorry.") 
	End If	
	If result <> 1 Then End

	;Clear the front and backbuffer
	ClsBB() : FlipBB()
	ClsBB() : FlipBB()	
	UpdateMouseBB() ;update the mouse position
	If ColorDepth = 16 ;record format used if in 16 bit mode
		screenFormat16Bit = CallDLL(dllFileName$,"Get16BitFormatBB")
	End If
	
	;In Blitz Plus the native Blitz mouse is enabled
	If BlitzPlusVersion = True CallDLL(dllFileName$,"UnlockBlitzMouseBB")
	
End Function

;Returns a pointer to the current drawing buffer
Function GraphicsBufferBB()
	Return CallDLL(dllFileName$,"GraphicsBufferBB")
End Function

;Returns the color depth of the current graphics mode (8,16,24,32)
Function GraphicsDepthBB()
	Return ColorDepth
End Function

;Returns the height, in pixels, of the current graphics mode.
Function GraphicsHeightBB()
	Return ScreenHeight
End Function

;Returns the width, in pixels, of the current graphics mode.
Function GraphicsWidthBB()
	Return ScreenWidth
End Function

;Sets the x and y handles of an image
;-Example = Draw3DImage.bb demo.
Function HandleImageBB(imagePointer,x,y)
	PokeInt BankStartIn,0,imagePointer
	PokeInt BankStartIn,4,x
	PokeInt BankStartIn,8,y		
	Return CallDLL(dllFileName$,"HandleImageBB",BankStartIn)	 
End Function

;This function doesn't do much since its functionality is 
;basically included in SetBufferBB. It is included here for
;compatibility's sake.
Function ImageBufferBB(imagePointer)
	Return imagePointer
End Function

;Returns the height in pixels of a loaded image.
Function ImageHeightBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer
	Return CallDLL(dllFileName$,"ImageHeightBB",BankStartIn)	 
End Function

;Checks whether two images have collided. It considers
;image masking/transparency and only registers a collision when
;non-masked pixels of the two images overlap one another.
;	This command is much faster when the images are loaded 
;into RAM rather than VRAM. Since VRAM images draw faster, 
;if you have a large image that you do a lot of checking on,
;it might be worth loading two separate copies of each image - one in
;VRAM for drawing, and one in RAM for image collisions. The
;check will be almost as fast if only one of the two images is
;loaded into RAM -- you should list that one first.
;	The ImageOverlapBB function is faster still, and should be
;used instead if you don't need pixel-perfect collision detection
;or your images are rectangular, non-transparent blocks.
;	This command works, but is slow, with unrotated or scaled 
;3D enabled images. They should be listed second if possible, 
;or a second non-3D image should be created and used for 
;collision purposes.
;- Example = ImagesCollide demo
Function ImagesCollideBB(imagePointer1,x1,y1,frame1,imagePointer2,x2,y2,frame2)
	PokeInt BankStartIn,0,imagePointer1
	PokeInt BankStartIn,4,x1
	PokeInt BankStartIn,8,y1
	PokeInt BankStartIn,12,frame1
	PokeInt BankStartIn,16,imagePointer2	
	PokeInt BankStartIn,20,x2
	PokeInt BankStartIn,24,y2
	PokeInt BankStartIn,28,frame2	
	Return CallDLL(dllFileName$,"ImagesCollideBB",BankStartIn)		
End Function

;Checks whether two images overlap without considering
;pixel transparency. This command works fine for both normal
;images and unrotated, unscaled 3D-enabled images.
;	Bug Note: Doesn't currently work with animated images.
;- Example = ImagesCollide demo
Function ImagesOverlapBB(imagePointer1,x1,y1,imagePointer2,x2,y2)
	PokeInt BankStartIn,0,imagePointer1
	PokeInt BankStartIn,4,x1
	PokeInt BankStartIn,8,y1
	PokeInt BankStartIn,12,imagePointer2	
	PokeInt BankStartIn,16,x2
	PokeInt BankStartIn,20,y2
	Return CallDLL(dllFileName$,"ImagesOverlapBB",BankStartIn)					 
End Function

;Returns the width in pixels of a loaded image.
Function ImageWidthBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer
	Return CallDLL(dllFileName$,"ImageWidthBB",BankStartIn)	 
End Function		

;Returns the image's x handle
Function ImageXHandleBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer
	Return CallDLL(dllFileName$,"ImageXHandleBB",BankStartIn)	 
End Function	

;Returns the image's y handle
Function ImageYHandleBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer
	Return CallDLL(dllFileName$,"ImageYHandleBB",BankStartIn)	 
End Function		

;Draws a line at the specified location using the current pen.
;- Example = GUI Commands demo
Function LineBB(x1,y1,x2,y2)
	PokeInt BankStartIn,0,x1  	
	PokeInt BankStartIn,4,y1  	
	PokeInt BankStartIn,8,x2  	
	PokeInt BankStartIn,12,y2	  	
	Return CallDLL(dllFileName$,"LineBB",BankStartIn)	
End Function

;Loads a 3D-enabled sprite or image. Such images are drawn using
;the Draw3DImageBB command and can do real time rotations, scaling, 
;and alpha blending. 
;	They are not currently compatible with 8 bit (paletted) display
;modes, however. Also, the loaded 3D-enabled image should be square,
;and its width and height both a power of 2. Example dimensions are
;32x32, 64x64, 128x128, 256x256, etc. Some video cards may not be 
;able to load images much larger than that.
;- Example = Draw3DImage.bb demo
Function Load3DImageBB(fileName$)
	If FileType(fileName$) <> 1 Then Return		
		
	;BMP Loader or External Loader
	If gImageLoader = 0 Or gImageLoader = 2
		StringToAsciiBB(fileName$,BankStartIn,0)
		imagePointer = CallDLL(dllFileName$,"Load3DImageBB",BankStartIn)	
		Return imagePointer	
		
	;Blitz Loader (png/jpg)
	Else If gImageLoader = 1 
		If ColorDepth = 8 Then Return ;doesn't work in 8-bit mode	
		blitzImage = LoadImage(fileName$)
		width = ImageWidth (blitzImage)
		height = ImageHeight (blitzImage)		
		image = CreateImageBB(width,height,loadTo)	
		SetBuffer ImageBuffer (blitzImage)
		LockBuffer ImageBuffer (blitzImage)		
		LockBufferWritingBB(image,width*height)			
		For xx = 0 To width-1 ;one less than image width since starting at 0
		For yy = 0 To height-1 ;one less than image height since starting at 0
			rgb = ReadPixelFast(xx,yy)		
			WritePixelFastBB(xx,yy,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
		Next
		Next
		UnlockBufferWritingBB(image)					
		UnlockBuffer ImageBuffer(blitzImage)	
		FreeImage (blitzImage)	
	
		;Now we transform it into a 3D image
		imageCopy2Dto3D = CopyImageBB(image,0,1)
		FreeImageBB(image)
		image = imageCopy2Dto3D
		Return image		
	End If	
End Function


;Load an animated image from disk into RAM or VRAM.
;Loaded images can be simple horizontal strips or 
;multi-level "stacked" strips.
;- loadTo = 0 for VRAM, 1 for RAM.
;Example = Draw Animated Image.bb
Function LoadAnimImageBB(fileName$,width,height,firstFrame,count,loadTo=0)
	If loadTo > 1 Then Return
	If FileType(fileName$) <> 1 Then Return	
	
	;BMP Loader or External Loader
	If gImageLoader = 0 Or gImageLoader = 2
		PokeInt BankStartIn,0,loadTo	
		PokeInt BankStartIn,4,width	
		PokeInt BankStartIn,8,height
		PokeInt BankStartIn,12,firstFrame		
		PokeInt BankStartIn,16,count
		StringToAsciiBB(fileName$,BankStartIn,20)			
		imagePointer = CallDLL(dllFileName$,"LoadImageBB",BankStartIn)	
		Return imagePointer
		
	;Blitz Loader (png/jpg)
	Else If gImageLoader = 1
		If ColorDepth = 8 Then Return ;doesn't work in 8-bit mode	
		blitzImage = LoadImage(fileName$)
		blitzImageWidth = ImageWidth (blitzImage)
		blitzImageHeight = ImageHeight (blitzImage)		
		SetBuffer ImageBuffer (blitzImage)
		LockBuffer ImageBuffer (blitzImage)		
		
		image = CreateImageBB(blitzImageWidth,blitzImageHeight,loadTo)			
		LockBufferWritingBB(image,blitzImageWidth*blitzImageHeight)			
		For xx = 0 To blitzImageWidth-1 ;one less than image width since starting at 0
		For yy = 0 To blitzImageHeight-1 ;one less than image height since starting at 0
			rgb = ReadPixelFast(xx,yy)		
			WritePixelFastBB(xx,yy,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
		Next
		Next
		UnlockBufferWritingBB(image)					

		UnlockBuffer ImageBuffer(blitzImage)	
		FreeImage (blitzImage)	

		;Make it an animated image
		PokeInt BankStartIn,0,image	
		PokeInt BankStartIn,4,width	
		PokeInt BankStartIn,8,height
		PokeInt BankStartIn,12,firstFrame		
		PokeInt BankStartIn,16,count		
		result = CallDLL(dllFileName$,"MakeAnimatedBB",BankStartIn)
		Return image
	End If
End Function	

;Load an image from disk into RAM or VRAM. Images drawn from VRAM 
;are typically drawn about twice as fast as images drawn from VRAM. 
;It is generally a good idea to load all of your graphics into VRAM
;if you can. If you have more graphics than available VRAM,
;load those graphics that are rarely used into RAM.
;- Use loadTo = 0 for VRAM, 1 for RAM.
;Example = DrawImage.bb demo.
Function LoadImageBB(fileName$,loadTo=0)
	If loadTo > 1 Then Return
	If FileType(fileName$) <> 1 Then Return	

	;BMP Loader or External Loader
	If gImageLoader = 0 Or gImageLoader = 2
		PokeInt BankStartIn,0,loadTo 
		PokeInt BankStartIn,4,0  ;not an animated image	
		PokeInt BankStartIn,8,0  ;not an animated image	
		PokeInt BankStartIn,12,0 ;not an animated image			
		PokeInt BankStartIn,16,1 ;not an animated image	
		StringToAsciiBB(fileName$,BankStartIn,20)			
		imagePointer = CallDLL(dllFileName$,"LoadImageBB",BankStartIn)	
		Return imagePointer		

	;Blitz Loader (png/jpg)
	Else If gImageLoader = 1 
		If ColorDepth = 8 Then Return ;doesn't work in 8-bit mode	
		blitzImage = LoadImage(fileName$)
		width = ImageWidth (blitzImage)
		height = ImageHeight (blitzImage)		
		image = CreateImageBB(width,height,loadTo)	
		SetBuffer ImageBuffer (blitzImage)
		LockBuffer ImageBuffer (blitzImage)		
		LockBufferWritingBB(image,width*height)			
		For xx = 0 To width-1 ;one less than image width since starting at 0
		For yy = 0 To height-1 ;one less than image height since starting at 0
			rgb = ReadPixelFast(xx,yy)		
			WritePixelFastBB(xx,yy,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))	
		Next
		Next
		UnlockBufferWritingBB(image)					
		UnlockBuffer ImageBuffer(blitzImage)	
		FreeImage (blitzImage)	
		Return image	
	End If
End Function


;This command prepares an image for fast pixel reading. It 
;actually copies the image over to a Blitz bank called
;pixelReadBank, where it can be read using ReadPixelFastBB.
;If you only need to read a few pixels from an image, it
;will probably be faster to use the ReadPixelBB command
;instead.
;	By default LockBufferReadingBB(imagePointer) will copy the
;entire image over. LockBufferReadingBB(imagePointer,0,0,100,150) 
;or something similar will only copy the specified portion of the image.
;This is faster and useful when you only want a portion of the target
;image. On the other hand, the ReadPixelFast command will consider
;the point (0,0) to be the upper left corner of the portion that was
;read.
;	You can read the front buffer or backbuffer using this command
;by putting BackBufferBB or FrontBufferBB in the imagePointer parameter.
;When reading either one, you must specify x,y,width, and height. Even 
;if you are reading a small portion of one of these buffers,
;it will still be pretty slow.
;- Example = WritePixelFast.bb demo.
Function LockBufferReadingBB(imagePointer,x=-1,y=-1,width=-1,height=-1)
	If imagePointer < 3 ;if locking the front or backbuffer
		If x = -1 Then Return -10 ;can't lock the whole buffer
		thisImageWidth = ScreenWidth
		thisImageHeight = 	ScreenHeight	
	Else ;if locking an image
		PokeInt BankStartIn,0,imagePointer 	
		thisImageWidth = CallDLL(dllFileName$,"ImageWidthBB",BankStartIn)
		thisImageHeight = CallDLL(dllFileName$,"ImageHeightBB",BankStartIn)
	End If
	If x = -1 ;if reading the whole image
		ResizeBank pixelReadBank,thisImageWidth*thisImageHeight*5+20
		PokeInt BankStartIn,0,imagePointer 		
		PokeInt BankStartIn,4,-1 			
	Else	;if reading only part of the surface, use the specified coordinates
		If x < 0 Then Return -20
		If y < 0 Then Return -30
		If x+width > thisImageWidth Then Return -40
		If y+height > thisImageHeight Then Return -50
		ResizeBank pixelReadBank,width*height*5					
		PokeInt BankStartIn,0,imagePointer 		
		PokeInt BankStartIn,4,x 			
		PokeInt BankStartIn,8,y 
		PokeInt BankStartIn,12,width 
		PokeInt BankStartIn,16,height 						
	End If
	ReadPixelPitch = CallDLL(dllFileName$,"ReadPixelFastBB",BankStartIn,pixelReadBank)
	Return ReadPixelPitch
End Function	

;This command prepares an image for fast pixel writing.
;It does this by setting up a Blitz bank called pixelWriteBank, 
;which is then passed to the dll when the UnlockBufferWritingBB
;command is called. No pixels are actually written to the image
;until then.
;   Please note that the second parameter isn't needed if the 
;target image is less than 200x200 pixels or less than 40000
;pixels are being written.
;- Example = WritePixelFast.bb demo.
Function LockBufferWritingBB(imagePointer,numberOfPixels=40000)
	If numberOfPixels > 40000 
		ResizeBank pixelWriteBank, (numberOfPixels+1)*20
	Else
		ResizeBank pixelWriteBank, (40001)*20	
	End If	
	PokeInt pixelWriteBank,0,imagePointer ;pointer to image to be written to	
	PokeInt pixelWriteBank,4,0 ;zero out number of pixels written/read so far		
End Function	

;Masks an image with the specified rgb color. Unlike Blitz, images 
;are not given an automatic default mask of 0,0,0. This makes 
;DrawImageBB a bit faster for images that are not masked. If you 
;want it to be automatically masked just like in Blitz, just use 
;AutoMaskImageBB(0,0,0).
;    If you are in 8 bit graphics mode, and if you happen to have more
;than one palette entry for the chosen mask color, the mask might not
;work correctly because MaskImageBB might choose the wrong palette 
;entry.
;Example = DrawImage.bb demo
Function MaskImageBB(imagePointer,r=0,g=0,b=0)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,r  	
	PokeInt BankStartIn,8,g  
	PokeInt BankStartIn,12,b	
	Return CallDLL(dllFileName$,"MaskImageBB",BankStartIn)	
End Function	

;This function masks an image by choosing a pixel in the image
;as the mask color rather than using an rgb value. This method
;can be more accurate if you do a lot of 2D-3D image copying back
;and forth because on some systems the format of the two types
;of images are different, resulting in slight color changes that
;can mess up masking.
;	The default pixel is the one at (0,0), so you can just use 
;MaskImagePixelBB(imagePointer) if that pixel is the mask color.
Function MaskImagePixelBB(imagePointer,x=0,y=0)
	SetBufferBB(imagePointer)
	rgb = ReadPixelBB(x,y)
	DebugLog "MaskImagePixelBB. Color at (0,0) = " + rgb
	DebugLog "-"+GetR_BB(rgb)
	DebugLog "-"+GetG_BB(rgb)
	DebugLog "-"+GetB_BB(rgb)		
	MaskImageBB(imagePointer,GetR_BB(rgb),GetG_BB(rgb),GetB_BB(rgb))
	DebugLog "-- " + GetValueBB(0)
	SetBufferBB(BackBufferBB)
End Function

;Creates a windows message box. Will not work while in full screen 
;game mode. Used for error messages either before or after launching.
Function MessageBoxBB(programTitle$,MessageBoxTitle$,boxText$)
	result = StringToAsciiBB(programTitle$,BankStartIn,0)
	result = StringToAsciiBB(MessageBoxTitle$,BankStartIn,result)
	result = StringToAsciiBB(boxText$,BankStartIn,result)
	CallDLL(dllFileName$,"MessageBoxBB",BankStartIn)
End Function

;Sets the handle of an image to its center.
;Example = Draw Animated Image.bb demo.
Function MidHandleBB(imagePointer)
	PokeInt BankStartIn,0,imagePointer	
	Return CallDLL(dllFileName$,"MidHandleBB",BankStartIn)	 
End Function

;Internal function used to update the MouseHit1BB and
;MouseHit2BB global variables used by MouseHitBB. This
;function is called every frame by the FlipBB command.
;If that command is not used, and you want to use the
;MouseHit command, you should call this command
;directly fairly often because the buffer can only hold 10 items
;at a time, and you will lose newer data when the buffer is full.
Function MouseBufferCheckBB()	
	bufferedItems = CallDLL(dllFileName$,"MouseHitBB",BankStartIn,BankStartOut)	 
	MouseHit1BB = MouseHit1BB+PeekInt(BankStartOut,0)	
	MouseHit2BB = MouseHit2BB+PeekInt(BankStartOut,4)
	Return bufferedItems ;used for testing purposes	
End Function

;Returns whether the specified mouse button is down. This info
;is updated by the FlipBB command, which is usually plenty. If
;you want to update it manually, call UpdateMouseBB()
;before using this command.
;	And, as is probably obvious from the contents of this
;function, it is faster just to use the LeftMouseButton or
;RightMouseButton global variables directly, which are
;updated by updated by FlipBB and UpdateMouseBB(),
;rather than using this function. If those values are 1,
;the selected button is down. If 0, they are up.
;Button: 1 = left, 2 = right
;	Please note that the native Blitz mouse commands work fine
;in Blitx Plus.
;- Example = Mouse demo
Function MouseDownBB(button=1)	
	If button=1 Return LeftMouseButton	 
	If button=2 Return RightMouseButton	 	
End Function

;Returns the number of times that either the selected button,
;either the left (1) or right (2), has been hit since the
;last time this command was called.
;	Please note that the native Blitz mouse commands work fine
;in Blitx Plus.
;- Example = Mouse demo
Function MouseHitBB(button)
	If button=1 
		temp = MouseHit1BB
		MouseHit1BB = 0
		Return temp
	Else	
		temp = MouseHit2BB
		MouseHit2BB = 0
		Return temp
	End If	
End Function

;Returns whether the specified mouse button is up. This info
;is updated by the FlipBB command, which is usually plenty. If
;you want to update it manually, call UpdateMouseBB()
;before using this command.
;	And, as is probably obvious from the contents of this
;function, it is faster just to use the LeftMouseButton or
;RightMouseButton global variables directly, which are
;updated by updated by FlipBB and UpdateMouseBB(),
;rather than using this function. If those values are 1,
;the selected button is down. If 0, they are up.
;	Please note that the native Blitz mouse commands work fine
;in Blitz Plus.
;Button: 1 = left, 2 = right
Function MouseUpBB(button=1)	
	If button=1 Return 1-LeftMouseButton	 
	If button=2 Return 1-RightMouseButton			 	
End Function

;Returns the "x" position of the mouse. The current position 
;of the mouse is updated automatically every time the FlipBB
;command is called, which is usually plenty. To get this
;information manually, call the UpdateMouseBB() command
;before using this command.
;	And, as is probably obvious from the contents of this
;function, it is faster just to use the MouseCursorX global
;variable, which is updated by FlipBB and UpdateMouseBB(), 
;rather than this function.
;	Please note that the native Blitz mouse commands work fine
;in Blitz Plus.
Function MouseXBB()	
	Return MouseCursorX	 
End Function

;Returns the "y" position of the mouse. The current position 
;of the mouse is updated automatically every time the FlipBB
;command is called, which is usually plenty. To get this
;information manually, call the UpdateMouseBB() command
;before using this command.
;	And, as is probably obvious from the contents of this
;function, it is faster just to use the MouseCursorY global
;variable, which is updated by FlipBB and UpdateMouseBB(), 
;rather than this function.
;	Please note that the native Blitz mouse commands work fine
;in Blitz Plus.
Function MouseYBB()	
	Return MouseCursorY	 
End Function

;Moves the mouse to the selected spot on the screen.
;	Please note that the native Blitz mouse commands work fine
;in Blitz Plus.
Function MoveMouseBB(x,y)	
	PokeInt BankStartIn,0,x
	PokeInt BankStartIn,4,y	
	CallDLL(dllFileName$,"MoveMouseBB",BankStartIn)	
	UpdateMouseBB();update global variables	 	
End Function

;Draws an oval or circle at the designated location.
;Solid = 0 draws empty oval (no fill), 1 = color filled
;- Example = GUI Commands demo
Function OvalBB(x,y,width,height,solid=0)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y  	
	PokeInt BankStartIn,8,width  	
	PokeInt BankStartIn,12,height
	PokeInt BankStartIn,16,solid	  	
	Return CallDLL(dllFileName$,"OvalBB",BankStartIn)	
End Function

;Draws a single pixel at the designated location.
;This command is slow in 8 bit mode because it plots
;the nearest palette entry, which takes time to figure out. 
;Unless you need to do that, use WritePixel instead.
;	This command does not work with 3D images. If you need
;to modify a 3D image, create a 2D image, read/write on that,
;and then use DrawImageBB. See Copy 2D to 3D demo.
;	When using this command on animated images, please note that
;the x/y values are relative to the entire image, not a particular
;frame.
;- Example = GUI Commands demo
Function PlotBB(x,y,r,g,b)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y  	
	PokeInt BankStartIn,8,r  	
	PokeInt BankStartIn,12,g
	PokeInt BankStartIn,16,b	  	
	Return CallDLL(dllFileName$,"PlotBB",BankStartIn)	
End Function

;Read a palette entry's combined RGB value. Palettes are only used
;in 8 bit (256 color) mode. Palette entries are color codes which
;are used instead of RGB values, and range in value from 0-255. 
;To read the palette entry used in an image at a certain location,
;use ReadPixel or ReadPixelFast.
;- Example = Palettes demo
Function ReadPaletteRGB_BB(paletteEntryNumber)
	PokeByte BankStartIn,0,paletteEntryNumber 	
	CallDLL(dllFileName$,"ReadPaletteBB",BankStartIn,BankStartOut)
	r = PeekByte(BankStartOut,0)			
	g = PeekByte(BankStartOut,1)
	b = PeekByte(BankStartOut,2)	
	Return GetRGB_BB(r,g,b)
End Function

;Read a palette entry's red value. Palettes are only used
;in 8 bit (256 color) mode. Palette entries are color codes which
;are used instead of RGB values, and range in value from 0-255. 
;To read the palette entry used in an image at a certain location,
;use ReadPixel or ReadPixelFast.
;- Example = Palettes demo
Function ReadPaletteR_BB(paletteEntryNumber)
	PokeByte BankStartIn,0,paletteEntryNumber 	
	CallDLL(dllFileName$,"ReadPaletteBB",BankStartIn,BankStartOut)
	Return PeekByte(BankStartOut,0)			
End Function	

;Read a palette entry's green value. Palettes are only used
;in 8 bit (256 color) mode. Palette entries are color codes which
;are used instead of RGB values, and range in value from 0-255. 
;To read the palette entry used in an image at a certain location,
;use ReadPixel or ReadPixelFast.
;- Example = Palettes demo
Function ReadPaletteG_BB(paletteEntryNumber)
	PokeByte BankStartIn,0,paletteEntryNumber 	
	CallDLL(dllFileName$,"ReadPaletteBB",BankStartIn,BankStartOut)
	Return PeekByte(BankStartOut,1)			
End Function	

;Read a palette entry's blue value. Palettes are only used
;in 8 bit (256 color) mode. Palette entries are color codes which
;are used instead of RGB values, and range in value from 0-255. 
;To read the palette entry used in an image at a certain location,
;use ReadPixel or ReadPixelFast.
;- Example = Palettes demo
Function ReadPaletteB_BB(paletteEntryNumber)
	PokeByte  BankStartIn,0,paletteEntryNumber 	
	CallDLL(dllFileName$,"ReadPaletteBB",BankStartIn,BankStartOut)
	Return PeekByte(BankStartOut,2)			
End Function	

;Read the color of a single pixel in the current drawing buffer 
;(set with SetBufferBB). If in 8 bit mode, this will be a palette 
;entry. Otherwise it will be a rgb value. You can convert
;palette entries to rgb using ReadPaletteRGB_BB.
;	This command does not work with 3D images. If you need
;to modify a 3D image, create a 2D image, read/write on that,
;and then use DrawImageBB. See Copy 2D to 3D demo.
;- Example = ReadPixel demo
Function ReadPixelBB(x,y)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y  	
	CallDLL(dllFileName$,"ReadPixelBB",BankStartIn,BankStartOut)	
	If ColorDepth = 8
		Return PeekByte(BankStartOut,0)					
	Else If ColorDepth = 16
		If screenFormat16Bit = 555			
			r = 8*(PeekShort(BankStartOut,0) Shr 10 And %11111) ;pos 10-14
			g = 8*(PeekShort(BankStartOut,0) Shr 5 And %11111) ;pos 5-9
			b = 8*(PeekShort(BankStartOut,0) And %11111);pos 0-4
		Else If screenFormat16Bit = 565
			r = 8*(PeekShort(BankStartOut,0) Shr 11 And %11111) ;pos 11-15
			g = 4*(PeekShort(BankStartOut,0) Shr 5 And %111111) ;pos 5-10
			b = 8*(PeekShort(BankStartOut,0) And %11111);pos 0-4	
		End If
		Return GetRGB_BB(r,g,b)				
	Else If ColorDepth = 24 Or ColorDepth = 32		
		r = PeekInt(BankStartOut,0) Shr 16 And %11111111
		g = PeekInt(BankStartOut,0) Shr 8 And %11111111	
		b = PeekInt(BankStartOut,0) And %11111111
		Return GetRGB_BB(r,g,b)			
	End If	
End Function	

;Read the color of a pixel in the image specified in the 
;LockBufferReading command. If in 8 bit mode, this
;will be a palette entry. Otherwise it will be a rgb value.
;Also, if only a portion of the target image was locked for
;reading, the x and y used here will be relative to the x
;and y used in the LockBufferReading command.
;	This command does not work with 3D images. If you need
;to modify a 3D image, create a 2D image, read/write on that,
;and then use DrawImageBB. See Copy 2D to 3D demo.
;- Example = WritePixelFast demo
Function ReadPixelFastBB(x,y)
	If ColorDepth = 8 ;if in 8 bit mode, return a palette entry
		Return PeekByte(pixelReadBank,x+y*ReadPixelPitch) 					
	Else If ColorDepth = 16		
		If screenFormat16Bit = 555
			r = 8*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) Shr 10 And %11111) ;pos 10-14
			g = 8*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) Shr 5 And %11111) ;pos 5-9
			b = 8*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) And %11111);pos 0-4
		Else If screenFormat16Bit = 565
			r = 8*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) Shr 11 And %11111) ;pos 11-15
			g = 4*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) Shr 5 And %111111) ;pos 5-10
			b = 8*(PeekShort(pixelReadBank,2*x+y*ReadPixelPitch) And %11111);pos 0-4	
		End If
		Return GetRGB_BB(r,g,b)					
	Else If ColorDepth = 24
		Return PeekInt(pixelReadBank,3*x+y*ReadPixelPitch) And %111111111111111111111111	
	Else If ColorDepth = 32
		Return PeekInt(pixelReadBank,4*x+y*ReadPixelPitch) And %111111111111111111111111	
	End If		
End Function	

;Draws a square or rectangle at the designated location.
;Solid = 0 draws empty oval (no fill), 1 = color filled.
;- Example = GUI Commands demo
Function RectBB(x,y,width,height,solid=0)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y  	
	PokeInt BankStartIn,8,width  	
	PokeInt BankStartIn,12,height
	PokeInt BankStartIn,16,solid	  	
	Return CallDLL(dllFileName$,"RectBB",BankStartIn)	
End Function

;Resizes an image to a new size using pixel
;dimensions. This command will not work on 3D-enabled images or 
;animated images created with LoadAnimImageBB.
;Example = Scale n Rotate.bb demo (uses similar ScaleImageBB command).
Function ResizeImageBB(imagePointer,width,height)
	PokeInt BankStartIn,0,imagePointer
	PokeInt BankStartIn,4,width
	PokeInt BankStartIn,8,height
	Return CallDLL(dllFileName$,"ResizeImageBB",BankStartIn)
End Function

;Rotates an existing image by 90, 180, or 270 degrees in a 
;clockwise direction. This is too slow for real time rotations.
;For real-time rotations, use a 3D-enabled image and Draw3DImageBB. This
;command will not work on 3D-enabled images or animated images created
;with LoadAnimImageBB.
;- Example = Scale n Rotate.bb demo
Function RotateImageBB(imagePointer,angle)
	If angle = 180 ;its faster to do a mode 2 FlipImage if it's a 180 degree rotate
		FlipImageBB(imagePointer,mode=2)	
		Return
	End If
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,angle  	
	Return CallDLL(dllFileName$,"RotateImageBB",BankStartIn)	
End Function

;Saves the designated image as a bmp file. Use
;BackBufferBB or FrontBufferBB to save those respective
;buffers.
;- Example = DrawImage.bb demo.
Function SaveBufferBB(imagePointer,fileName$)
	SetBufferBB(imagePointer)
	StringToAsciiBB(fileName$,BankStartIn)
	CallDLL(dllFileName$,"SaveBufferBB",BankStartIn)
	SetBufferBB(BackBufferBB)	
End Function

;Same as SaveBufferBB.
;- Example = Palettes.bb demo.
Function SaveImageBB(imagePointer,fileName$)
	SaveBufferBB(imagePointer,fileName$)
End Function

;Scales an image to a new size using fractions.
;i.e., 0.5 = 50%, 1 = 100%, 2 = 200%, etc.
;This command will not work on 3D-enabled images or 
;animated images created with LoadAnimImageBB.
;	Bug Note: I've noticed that this command works
;strangely in 16 bit mode on my own 4 year old
;laptop. This started after I downloaded a newer
;driver that wasn't really the right one for the
;machine, but I thought I'd mention it. It isn't
;a problem on other machines that I've tried.
;- Example = Scale n Rotate demo
Function ScaleImageBB(imagePointer,xScale#,yScale#)
	PokeInt BankStartIn,0,imagePointer
	width = ImageWidthBB(imagePointer)*xScale
	height = ImageHeightBB(imagePointer)*yScale	
	PokeInt BankStartIn,4,width
	PokeInt BankStartIn,8,height	
	Return CallDLL(dllFileName$,"ResizeImageBB",BankStartIn)
End Function

;Applies an alpha mask to a 3D image. The image used as an alpha 
;mask must have the same dimensions as the image being masked. 
;Normally it would be a gray-scale image, but it can have any colors
;really. The red color values in the mask will be interpreted
;as alpha and applied to the masked image.
Function Set3DAlphaTextureBB(imagePointer,alphaTexture)
	PokeInt BankStartIn,0,imagePointer
	PokeInt BankStartIn,4,alphaTexture	
	Return CallDLL(dllFileName$,"Set3DAlphaTextureBB",BankStartIn)	
End Function

;Sets the fill color and pattern for ovals and rectangles
;Pattern =  false - use a solid color
;			true - use a non-solid pattern of the specified style
;Styles: 0 = b diagonal, 1 = cross, 2 = diagonal cross, 3 = f diagonal,
;	4 = horizontal, 5 = vertical.
;	In 8-bit mode this command is slow -- too slow for real-time
;use. This is because the command needs to search through the entire
;palette to find the closest palette entry. Unfortunately, the 
;underlying Microsoft Windows commands being used can not use
;palette entries, which would be a lot faster.
Function SetBrushBB(r,g,b,pattern=False,style=0)
	PokeInt BankStartIn,0,r
	PokeInt BankStartIn,4,g
	PokeInt BankStartIn,8,b	
	PokeInt BankStartIn,12,pattern		
	PokeInt BankStartIn,16,style
	CallDLL(dllFileName$,"SetBrushBB",BankStartIn)
End Function

;Changes the current drawing buffer. The starting buffer is the back 
;buffer. This is different from Blitz, which sets the front buffer as
;the default starting buffer.
;   Please note that this command does not work with Draw3DImage or
;Draw3DQuad. 3D-enabled images can only be drawn on the backbuffer.
;- Examples: SetBufferBB(BackBufferBB), SetBufferBB(imageName)
Function SetBufferBB(pointer=2)
	PokeInt BankStartIn,0,pointer 	
	CallDLL(dllFileName$,"SetBufferBB",BankStartIn)	
End Function

;Sets the current font for use with the TextBB command.
;-Example = GUI Commands.bb demo.
Function SetFontBB(fontName$,typeSize,bold=False,italic=False,underline=False)
	PokeInt BankStartIn,0,typeSize 	
	PokeInt BankStartIn,4,bold  ;true/false
	PokeInt BankStartIn,8,italic ;true/false		
	PokeInt BankStartIn,12,underline  ;true/false		
	StringToAsciiBB(fontName$,BankStartIn,16)
	CallDLL(dllFileName$,"SetFontBB",BankStartIn)
	
	;Load and set this font using Blitz commands so that the Blitz StringWidth, 
	;StringHeight, FontWidth and FontHeight commands work properly. 
	;This font isn't otherwise used for anything.
	If currentFontBB <> 0 Then FreeFont currentFontBB
	currentFontBB = LoadFont(fontName$,typeSize,bold,italic,underline)
	If BlitzPlusVersion = False SetFont currentFontBB 
End Function

;Sets the color for text drawn with the TextBB command. 
;Default starting color is white.
;	In 8-bit mode this command is slow -- too slow for real-time
;use. This is because the command needs to search through the entire
;palette to find the closest palette entry. Unfortunately, the 
;underlying Microsoft Windows commands being used can not use
;palette entries, which would be a lot faster.
Function SetFontColorBB(r,g,b)
	PokeInt BankStartIn,0,r  	
	PokeInt BankStartIn,4,g  
	PokeInt BankStartIn,8,b 	
	Return CallDLL(dllFileName$,"SetFontColorBB",BankStartIn)	
End Function	

;This function sets the image loader that will be used by
;the LoadImageBB, Load3DImageBB, and LoadAnimImageBB commands.
;0 = Default BMP-only loader. Fast.
;1 = Blitz Loader for pngs, jpgs, and bmps. Slow.
;2 = External Loader for pngs, jpgs. Fast. Requires FreeImage.dll. 
;- Example/More Info = Image Loader.bb demo
Function SetImageLoaderBB(value)
	If value = 0 Then gImageLoader = 0
	If value = 1 Then gImageLoader = 1
	If value = 2 
		result = CallDLL(dllFileName$,"LoadFreeImageBB")
		If result = 0 gImageLoader = 1 ;FreeImage.dll load failed
		If result = 1 gImageLoader = 2	
	End If	
End Function	

;Sets the starting mouse speed. Starting speed is 2.
;Speed must be an integer. Higher numbers are faster.
Function SetMouseSpeedBB(speed)
	PokeInt BankStartIn,0,speed 
	CallDLL(dllFileName$,"SetMouseSpeedBB",BankStartIn)
End Function

;SetPaletteBB: Sets a 256 color palette after it has been changed using
;WritePaletteBB. The default settings reset the entire palette. You can 
;also just change a single entry in the palette or specify a range of
;entries, and only those palette entries will be changed. If the
;startingEntry plus entryCount go over 256, no change will occur.
;- Example = Palettes.bb demo.
Function SetPaletteBB(startingEntry=0,entryCount=255)
	PokeByte BankStartIn,0,startingEntry 
	PokeByte  BankStartIn,1,entryCount 				
	Return CallDLL(dllFileName$,"SetPaletteBB",BankStartIn)				
End Function	

;Sets the color for lines and the edges of ovals and rectangles
;Styles: 0 = solid, 1 = dash, 2 = dot, 3 = dash-dot, 4 = dash-dot-dot,
;	5 = null (transparent)
;Note that styles other than 0 will have a thickness of 1.
;	In 8-bit mode this command is slow -- too slow for real-time
;use. This is because the command needs to search through the entire
;palette to find the closest palette entry. Unfortunately, the 
;underlying Microsoft Windows commands being used can not use
;palette entries, which would be a lot faster.
;- Example = GUI Commands.bb demo.
Function SetPenBB(r,g,b,thickness=1,style=0)
	PokeInt BankStartIn,0,r 	
	PokeInt BankStartIn,4,g 
	PokeInt BankStartIn,8,b 			
	PokeInt BankStartIn,12,thickness 			
	PokeInt BankStartIn,16,style ;0-5
	CallDLL(dllFileName$,"SetPenBB",BankStartIn)
End Function

;Internal command that can be ignored. 
;Converts a Blitz string into a null terminated ASCII string stored in 
;a bank. This allows the string to be passed to the dll.
Function StringToAsciiBB(PassedString$,bank,startOfString=0)
	StringLength = Len( PassedString$ ) ;get the length of the passed string, and add 5. 4 the byte size of the int we will pass, and 1 for the NULL
	For character = startOfString To (startOfString+StringLength-1)
		Char$ = Left( PassedString$,1 ) ;get the left-most letter				
		PassedString$ = Right(PassedString$,Len( PassedString$ )-1) ;trim the letter off the front of the string
		PokeByte Bank,character,Asc( Char$ ) ;store the ascii value of the letter in the bank
	Next
	PokeByte Bank,startOfString+StringLength,0 ;store the null termination character
	startNextData = startOfString+StringLength+1
	Return startNextData ;return the point in the bank where further data entry may occur
End Function

;Prints a text string to the current drawing buffer. To change
;the current drawing buffer, use SetBufferBB.
;- Example = GUI Commands.bb demo.
Function TextBB(x,y,textString$)
	PokeInt BankStartIn,0,x  	
	PokeInt BankStartIn,4,y 	
	StringToAsciiBB(textString$,BankStartIn,8)
	CallDLL(dllFileName$,"TextBB",BankStartIn)
End Function

;Tints a 3D image with a given color. Default color is
;white, which is 255,255,255.
;- Example = Draw3DQuad.bb demo.
Function Tint3DImageBB(imagePointer,r,g,b)
	PokeInt BankStartIn,0,imagePointer 	
	PokeInt BankStartIn,4,r  	
	PokeInt BankStartIn,8,g  
	PokeInt BankStartIn,12,b	
	Return CallDLL(dllFileName$,"Tint3DImageBB",BankStartIn)	
End Function

;Get total physical system memory in bytes.
;- Example = Memory Leak demo
Function TotalSysMemBB()
	Return CallDLL(dllFileName$,"TotalSysMemBB")	
End Function	

;This function is not needed, and doesn't do anything.
;It's just here for full command compatibility with native Blitz.
Function UnlockBufferReadingBB(imagePointer)
	;does nothing
End Function

;"Unlocks" the image selected for fast pixel writing. Actually, this
;function calls the dll and sends it the pixelWriteBank for batch pixel 
;writing.
;- Example = WritePixelFast.bb demo.
Function UnlockBufferWritingBB(imagePointer)
	If imagePointer <> PeekInt (pixelWriteBank,0) Then Return ;error check image written to			
	Return CallDLL(dllFileName$,"WritePixelFastBB",pixelWriteBank)	
End Function

;Internal command that can be ignored. 
;Updates the current position and button status of the mouse using Direct Input.
;This function is called every time the FlipBB command is called, so you
;probably don't need to use it directly. Each of the four variables in the function
;is a global that is used by the MouseXBB/MouseYBB and MouseDownBB/
;MouseUpBB commands.
Function UpdateMouseBB()
	CallDLL(dllFileName$,"UpdateMouseBB",BankStartIn,BankStartOut)
	MouseCursorX  = PeekInt(BankStartOut,0)	
	MouseCursorY  = PeekInt(BankStartOut,4)	
	LeftMouseButton = PeekInt(BankStartOut,8)	
	RightMouseButton = PeekInt(BankStartOut,12)	
;	MouseXSpeedBB = Abs(PeekInt(BankStartOut,16))
;	MouseYSpeedBB = Abs(PeekInt(BankStartOut,20))
End Function

;Returns whether the game is currently active (1) or has been 
;minimized (0).
;- Example = Alt Tab.bb demo
Function WindowStatusBB()
	Return CallDLL(dllFileName$,"WindowStatusBB")
End Function

;Writes an RGB value to a given palette entry in a 256 color palette.
;(Please note that palettes are only used in 8 bit graphics mode.)
;This command will not actually change the palette itself. To change the
;palette, you need to use SetPaletteBB after you have changed the palette
;entries to whatever you want. 
;- Example = Palettes.bb demo
Function WritePaletteBB(paletteEntryNumber,r,g,b)
	PokeByte BankStartIn,0,paletteEntryNumber 
	PokeByte  BankStartIn,1,r 
	PokeByte  BankStartIn,2,g 
	PokeByte  BankStartIn,3,b 				
	CallDLL(dllFileName$,"WritePaletteBB",BankStartIn)		
End Function

;Writes an RGB value to a given spot in the current drawing buffer.
;If in 8 bit mode, only the "r" parameter is used and it is treated
;as a palette entry (0-255).
;	This command does not work with 3D images. If you need
;to modify a 3D image, create a 2D image, read/write on that,
;and then use DrawImageBB. See Copy 2D to 3D demo.
;	When using this command on animated images, please note that
;the x/y values are relative to the entire image, not a particular
;frame.
Function WritePixelBB(x,y,r,g=0,b=0)
	PokeInt BankStartIn,0,x 
	PokeInt BankStartIn,4,y	
	PokeInt BankStartIn,8,r		
	PokeInt BankStartIn,12,g	
	PokeInt BankStartIn,16, b 	
	Return CallDLL(dllFileName$,"WritePixelBB",BankStartIn)		
End Function

;Writes pixels on an image locked with LockBufferWritingBB. 
;If in 8 bit mode, only the "r" value of the rgb triplet is used, and
;it is treated as a palette entry. This command doesn't actually
;write any of the pixels until the UnlockBufferBB command is called.
;WritePixelFast is a bit faster on images saved in RAM.
;	This command does not work with 3D images. If you need
;to modify a 3D image, create a 2D image, read/write on that,
;and then use DrawImageBB. See Copy 2D to 3D demo.
;	When using this command on animated images, please note that
;the x/y values are relative to the entire image, not a particular
;frame.
;- Example = WritePixelFast demo
Function WritePixelFastBB(x,y,r,g=0,b=0)
	numberPixelsWritten = PeekInt(pixelWriteBank,4) + 1
	PokeInt pixelWriteBank,4,numberPixelsWritten
	If ColorDepth = 8
		PokeInt pixelWriteBank, numberPixelsWritten*20,x
		PokeInt pixelWriteBank, 4+numberPixelsWritten*20,y
		PokeInt pixelWriteBank, 8+numberPixelsWritten*20,r	;actually, a palette entry	
	Else
		PokeInt pixelWriteBank, numberPixelsWritten*20,x
		PokeInt pixelWriteBank, 4+numberPixelsWritten*20,y
		PokeInt pixelWriteBank, 8+numberPixelsWritten*20,r		
		PokeInt pixelWriteBank, 12+numberPixelsWritten*20,g	
		PokeInt pixelWriteBank, 16+numberPixelsWritten*20,b					
	End If	
End Function


	